package com.qmth.distributed.print.business.activiti.custom.listener;

import org.activiti.engine.delegate.event.ActivitiEvent;
import org.activiti.engine.delegate.event.ActivitiEventListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.Serializable;

/**
 * @Description: 全局监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/2/8
 */
@Component
public class ProcessEventListener implements ActivitiEventListener, Serializable {
    private final static Logger log = LoggerFactory.getLogger(ProcessEventListener.class);

    @Override
    public void onEvent(ActivitiEvent event) {
        switch (event.getType()) {
            case PROCESS_STARTED:
                // 流程启动
                log.info("流程启动_PROCESS_STARTED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
            case PROCESS_COMPLETED:
                // 流程结束
                log.info("流程结束_PROCESS_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
//            case ACTIVITY_COMPENSATE:
//                // 一个节点将要被补偿。事件包含了将要执行补偿的节点id。
//                log.info("ACTIVITY_COMPENSATE:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
            case ACTIVITY_COMPLETED:
                // 一个节点成功结束
                log.info("流程节点完成_ACTIVITY_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
//            case ACTIVITY_ERROR_RECEIVED:
//                // 一个节点收到了一个错误事件。在节点实际处理错误之前触发。 事件的activityId对应着处理错误的节点。 这个事件后续会是ACTIVITY_SIGNALLED或ACTIVITY_COMPLETE， 如果错误发送成功的话。
//                log.info("ACTIVITY_ERROR_RECEIVED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ACTIVITY_MESSAGE_RECEIVED:
//                // 一个节点收到了一个消息。在节点收到消息之前触发。收到后，会触发ACTIVITY_SIGNAL或ACTIVITY_STARTED，这会根据节点的类型（边界事件，事件子流程开始事件）
//                log.info("ACTIVITY_MESSAGE_RECEIVED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ACTIVITY_SIGNALED:
//                // 一个节点收到了一个信号
//                log.info("ACTIVITY_SIGNALED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ACTIVITY_STARTED:
//                // 一个节点开始执行
//                log.info("ACTIVITY_STARTED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case CUSTOM:
//                log.info("CUSTOM:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENGINE_CLOSED:
//                // 监听器监听的流程引擎已经关闭，不再接受API调用。
//                log.info("ENGINE_CLOSED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENGINE_CREATED:
//                // 监听器监听的流程引擎已经创建完毕，并准备好接受API调用。
//                log.info("ENGINE_CREATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_ACTIVATED:
//                // 激活了已存在的实体，实体包含在事件中。会被ProcessDefinitions, ProcessInstances 和 Tasks抛出。
//                log.info("ENTITY_ACTIVATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_CREATED:
//                // 创建了一个新实体。实体包含在事件中。
//                log.info("ENTITY_CREATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_DELETED:
//                // 删除了已存在的实体。实体包含在事件中
//                log.info("ENTITY_DELETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_INITIALIZED:
//                // 创建了一个新实体，初始化也完成了。如果这个实体的创建会包含子实体的创建，这个事件会在子实体都创建/初始化完成后被触发，这是与ENTITY_CREATED的区别。
//                log.info("ENTITY_INITIALIZED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_SUSPENDED:
//                // 暂停了已存在的实体。实体包含在事件中。会被ProcessDefinitions, ProcessInstances 和 Tasks抛出。
//                log.info("ENTITY_SUSPENDED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case ENTITY_UPDATED:
//                // 更新了已存在的实体。实体包含在事件中。
//                log.info("ENTITY_UPDATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case JOB_EXECUTION_FAILURE:
//                // 作业执行失败。作业和异常信息包含在事件中。
//                log.info("JOB_EXECUTION_FAILURE:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case JOB_EXECUTION_SUCCESS:
//                // 作业执行成功。job包含在事件中。
//                log.info("JOB_EXECUTION_SUCCESS:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case JOB_RETRIES_DECREMENTED:
//                // 因为作业执行失败，导致重试次数减少。作业包含在事件中。
//                log.info("JOB_RETRIES_DECREMENTED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case MEMBERSHIPS_DELETED:
//                // 所有成员被从一个组中删除。在成员删除之前触发这个事件，所以他们都是可以访问的。 因为性能方面的考虑，不会为每个成员触发单独的MEMBERSHIP_DELETED事件。
//                log.info("MEMBERSHIPS_DELETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case MEMBERSHIP_CREATED:
//                // 用户被添加到一个组里。事件包含了用户和组的id。
//                log.info("MEMBERSHIP_CREATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case MEMBERSHIP_DELETED:
//                // 用户被从一个组中删除。事件包含了用户和组的id。
//                log.info("MEMBERSHIP_DELETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case TASK_ASSIGNED:
//                // 任务被分配给了一个人员。事件包含任务。
//                log.info("TASK_ASSIGNED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
            case TASK_COMPLETED:
                // 任务被完成了。它会在ENTITY_DELETE事件之前触发。当任务是流程一部分时，事件会在流程继续运行之前， 后续事件将是ACTIVITY_COMPLETE，对应着完成任务的节点。
                log.info("流程任务完成_TASK_COMPLETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
//            case TIMER_FIRED:
//                // 触发了定时器。job包含在事件中。
//                log.info("TIMER_FIRED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case UNCAUGHT_BPMN_ERROR:
//                log.info("UNCAUGHT_BPMN_ERROR:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case VARIABLE_CREATED:
//                log.info("VARIABLE_CREATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case VARIABLE_DELETED:
//                log.info("VARIABLE_DELETED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
//            case VARIABLE_UPDATED:
//                log.info("VARIABLE_UPDATED:ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
//                break;
            default:
//                log.info("default->type:{},ProcessInstanceId:{},ExecutionId:{},ProcessDefinitionId:{}", event.getType(), event.getProcessInstanceId(), event.getExecutionId(), event.getProcessDefinitionId());
                break;
        }
    }

    @Override
    public boolean isFailOnException() {
        return false;
    }
}
