package com.qmth.distributed.print.business.bean.flow;

import com.qmth.distributed.print.business.enums.CustomFlowTypeEnum;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @Description: 自定义流程dto
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/1/21
 */
public class CustomFlowDto implements Serializable {

    @ApiModelProperty(value = "id")
    @NotBlank(message = "id不能为空")
    String id;

    @ApiModelProperty(value = "节点类型")
    @NotNull(message = "节点类型不能为空")
    CustomFlowTypeEnum type;

    @ApiModelProperty(value = "节点内容")
    String content;

    @ApiModelProperty(value = "width")
    Double w;

    @ApiModelProperty(value = "height")
    Double h;

    @ApiModelProperty(value = "x坐标轴")
    Double x;

    @ApiModelProperty(value = "y坐标轴")
    Double y;

    @ApiModelProperty(value = "属性")
    CustomFlowPropertyDto property;

    @ApiModelProperty(value = "流程属性id")
    String flowTaskId;

    @ApiModelProperty(value = "前置流程节点id集合")
    List<CustomFlowSequenceDto> beforeFlowTaskIds;

    @ApiModelProperty(value = "后置流程节点id集合")
    List<CustomFlowSequenceDto> afterFlowTaskIds;

    @ApiModelProperty(value = "流程步骤")
    Integer setup;

    public CustomFlowDto() {

    }

    public CustomFlowDto(String id, CustomFlowTypeEnum type, String content, String flowTaskId) {
        this.id = id;
        this.type = type;
        this.content = content;
        this.flowTaskId = flowTaskId;
    }

    public Integer getSetup() {
        return setup;
    }

    public void setSetup(Integer setup) {
        this.setup = setup;
    }

    public List<CustomFlowSequenceDto> getBeforeFlowTaskIds() {
        if (Objects.isNull(beforeFlowTaskIds)) {
            return new ArrayList<>();
        } else {
            return beforeFlowTaskIds;
        }
    }

    public void setBeforeFlowTaskIds(List<CustomFlowSequenceDto> beforeFlowTaskIds) {
        this.beforeFlowTaskIds = beforeFlowTaskIds;
    }

    public List<CustomFlowSequenceDto> getAfterFlowTaskIds() {
        return afterFlowTaskIds;
    }

    public void setAfterFlowTaskIds(List<CustomFlowSequenceDto> afterFlowTaskIds) {
        this.afterFlowTaskIds = afterFlowTaskIds;
    }

    public String getFlowTaskId() {
        return flowTaskId;
    }

    public void setFlowTaskId(String flowTaskId) {
        this.flowTaskId = flowTaskId;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public CustomFlowTypeEnum getType() {
        return type;
    }

    public void setType(CustomFlowTypeEnum type) {
        this.type = type;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public Double getW() {
        return w;
    }

    public void setW(Double w) {
        this.w = w;
    }

    public Double getH() {
        return h;
    }

    public void setH(Double h) {
        this.h = h;
    }

    public Double getX() {
        return x;
    }

    public void setX(Double x) {
        this.x = x;
    }

    public Double getY() {
        return y;
    }

    public void setY(Double y) {
        this.y = y;
    }

    public CustomFlowPropertyDto getProperty() {
        return property;
    }

    public void setProperty(CustomFlowPropertyDto property) {
        this.property = property;
    }
}
