package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.activiti.custom.service.DefaultInstanceConvertToMultiInstance;
import com.qmth.distributed.print.business.activiti.custom.service.MultiWorkFlow;
import com.qmth.distributed.print.business.bean.flow.*;
import com.qmth.distributed.print.business.bean.flow.link.FlowTaskLink;
import com.qmth.distributed.print.business.bean.flow.link.FlowTaskNode;
import com.qmth.distributed.print.business.bean.result.*;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.*;
import com.qmth.distributed.print.business.service.*;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.activiti.bpmn.BpmnAutoLayout;
import org.activiti.bpmn.model.Process;
import org.activiti.bpmn.model.*;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.impl.bpmn.behavior.MultiInstanceActivityBehavior;
import org.activiti.engine.impl.bpmn.behavior.ParallelMultiInstanceBehavior;
import org.activiti.engine.impl.bpmn.behavior.SequentialMultiInstanceBehavior;
import org.activiti.engine.repository.Deployment;
import org.activiti.engine.repository.DeploymentBuilder;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @Description: activiti service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/7/12
 */
@Service
public class ActivitiServiceImpl implements ActivitiService {
    private final static Logger log = LoggerFactory.getLogger(ActivitiServiceImpl.class);

    @Resource
    RuntimeService runtimeService;

    @Resource
    TaskService taskService;

    @Resource
    RepositoryService repositoryService;

    @Resource
    TFFlowApproveService tfFlowApproveService;

    @Resource
    TFFlowLogService tfFlowLogService;

    @Resource
    HistoryService historyService;

    @Resource
    ExamTaskService examTaskService;

//    @Resource
//    TFFlowApproveLogService tfFlowApproveLogService;

    @Resource
    RedisUtil redisUtil;

//    @Resource
//    TFFlowService tfFlowService;

    @Resource
    SysUserService sysUserService;

    @Resource
    ExamDetailCourseService examDetailCourseService;

    @Resource
    ExamDetailService examDetailService;

    @Resource
    TFFlowJoinService tfFlowJoinService;

    @Resource
    MultiWorkFlow multiWorkFlow;

    @Resource
    TFCustomFlowService tfCustomFlowService;

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    /**
     * 注册流程
     *
     * @param processFileName
     */
    @Override
    public void createDeployment(String processFileName) {
        DeploymentBuilder builder = repositoryService.createDeployment();
        builder.addClasspathResource(SystemConstant.PROCESSES + File.separator + processFileName);
        builder.deploy();
    }

    /**
     * 上传流程文件
     *
     * @param file
     */
    @Override
    public void uploadDeployment(MultipartFile file) throws IOException {
        DeploymentBuilder builder = repositoryService.createDeployment();
        builder.addInputStream(file.getOriginalFilename(), file.getInputStream());
        builder.deploy();
    }

    /**
     * 上传流程文件
     *
     * @param file
     */
    @Override
    public void uploadDeployment(File file) throws IOException {
        DeploymentBuilder builder = repositoryService.createDeployment();
        builder.addInputStream(file.getName(), new FileInputStream(file));
        builder.deploy();
    }

    /**
     * 根据流程key开启一个流程
     *
     * @param processKey
     * @param map
     */
    @Override
    @Transactional
    public ProcessInstance startActivity(String processKey, Map<String, Object> map) {
        ProcessInstance processInstance = null;
        if (Objects.nonNull(map) && map.size() > 0) {
            processInstance = runtimeService.startProcessInstanceByKey(processKey, map);
        } else {
            processInstance = runtimeService.startProcessInstanceByKey(processKey);
        }
        log.info("流程id:{},processInstanceId:{},processDefinitionId:{},processDefinitionKey:{}", processInstance.getId(), processInstance.getProcessInstanceId(), processInstance.getProcessDefinitionId(), processInstance.getProcessDefinitionKey());
        ProcessInstance finalProcessInstance = processInstance;
        map.computeIfAbsent(SystemConstant.FLOW_ID, v -> finalProcessInstance.getId());
        map.computeIfAbsent(SystemConstant.FLOW_STATUS, v -> FlowStatusEnum.START);
        this.flowApproveStart(map);
        return processInstance;
    }

    /**
     * 启动流程
     *
     * @param map
     * @return
     */
    @Override
    public ProcessInstance flowStart(Map<String, Object> map) {
        String id = (String) map.get(SystemConstant.ID);
        Map flowProcessVarMap = (Map) map.get(SystemConstant.FLOW_PROCESS_VAR_MAP);
        Map<String, CustomFlowVarDto> agginessMap = (Map<String, CustomFlowVarDto>) flowProcessVarMap.get(SystemConstant.AGGINESS_MAP);
        Map<String, Object> varMap = new HashMap<>();
        Gson gson = new Gson();
        for (Map.Entry<String, CustomFlowVarDto> entry : agginessMap.entrySet()) {
            CustomFlowVarDto customFlowVarDto = gson.fromJson(gson.toJson(entry.getValue()), CustomFlowVarDto.class);
            varMap.computeIfAbsent(customFlowVarDto.getFlowTaskVar(), s -> customFlowVarDto.getApproveIds().size() == 1 ? customFlowVarDto.getApproveIds().get(0) : customFlowVarDto.getApproveIds());
        }
        //启动流程
        ProcessInstance processInstance = varMap.size() > 0 ? runtimeService.startProcessInstanceByKey(id, varMap) : runtimeService.startProcessInstanceByKey(id);
        map.computeIfAbsent(SystemConstant.PROCESS_DEFINITION_ID, v -> processInstance.getProcessDefinitionId());
        map.computeIfAbsent(SystemConstant.FLOW_ID, v -> processInstance.getProcessInstanceId());
        map.computeIfAbsent(SystemConstant.FLOW_STATUS, v -> FlowStatusEnum.START);
        this.createFlowPicture(map);
        this.flowApproveStart(map);
        return processInstance;
    }

    /**
     * 创建流程图片
     *
     * @param map
     */
    protected void createFlowPicture(Map<String, Object> map) {
        String id = (String) map.get(SystemConstant.FLOW_ENTITY_ID);
        String processDefinitionId = (String) map.get(SystemConstant.PROCESS_DEFINITION_ID);
        String bpmnPngName = id + DefaultInstanceConvertToMultiInstance.FLOW_BPMN_PNG_NAME;

        //保存png图片(这一步可做可不做)
        InputStream processDiagram = repositoryService.getProcessDiagram(processDefinitionId);
        try {
            FileUtils.copyInputStreamToFile(processDiagram, new File(SystemConstant.TEMP_FILES_DIR + File.separator + DefaultInstanceConvertToMultiInstance.BPMN_NAME + File.separator + bpmnPngName));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 部署流程
     *
     * @param map
     * @return
     */
    protected Deployment flowDeployment(Map<String, Object> map) {
        String id = (String) map.get(SystemConstant.ID);
        BpmnModel model = (BpmnModel) map.get(SystemConstant.MODEL);
        Integer version = (Integer) map.get(SystemConstant.VERSION);

        //部署流程
        String bpmnName = id + DefaultInstanceConvertToMultiInstance.FLOW_BPMN_MODEL_NAME;
        String bpmnDeploymentName = id + DefaultInstanceConvertToMultiInstance.FLOW_BPMN_DEPLOYMENT_NAME;
        String bpmnProcessName = id + "_v" + version + DefaultInstanceConvertToMultiInstance.FLOW_BPMN_PROCESS_NAME;

        Deployment deployment = repositoryService.createDeployment().addBpmnModel(bpmnName, model).name(bpmnDeploymentName).deploy();
        map.remove(SystemConstant.MODEL);

        //保存xml文件(这一步可做可不做)
        InputStream processBpmn = repositoryService.getResourceAsStream(deployment.getId(), bpmnName);
        try {
            FileUtils.copyInputStreamToFile(processBpmn, new File(SystemConstant.TEMP_FILES_DIR + File.separator + DefaultInstanceConvertToMultiInstance.BPMN_NAME + File.separator + bpmnProcessName));
        } catch (IOException e) {
            e.printStackTrace();
        }
        return deployment;
    }

    /**
     * 根据流程id直接结束流程
     *
     * @param flowId
     */
    @Override
    @Transactional
    public void flowEnd(String flowId) {
        boolean lock = redisUtil.lock(SystemConstant.REDIS_LOCK_FLOW_PREFIX + flowId, SystemConstant.REDIS_LOCK_FLOW_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在终止中，请稍候再试！");
        }
        try {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            ExamTask examTask = examTaskService.findByFlowId(SystemConstant.convertIdToLong(flowId));
            //流程审批
            TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(SystemConstant.convertIdToLong(flowId), examTask.getSchoolId());
            TFFlowJoin tfFlowJoin = tfFlowJoinService.findByRootLevel(SystemConstant.convertIdToLong(flowId));
            if (Objects.isNull(tfFlowApprove)) {
                tfFlowApprove = new TFFlowApprove(sysUser.getSchoolId(), sysUser.getOrgId(), SystemConstant.convertIdToLong(flowId), sysUser.getId(), FlowStatusEnum.END, sysUser.getId());
            } else {
                tfFlowApprove.setStatus(FlowStatusEnum.END);
                tfFlowApprove.setSetup(FlowApproveSetupEnum.END.getSetup());
                tfFlowApprove.updateInfo(sysUser.getId());
            }
            tfFlowApproveService.saveOrUpdate(tfFlowApprove);

            //流程流水日志
            TFFlowLog tfFlowLog = new TFFlowLog(sysUser.getSchoolId(), sysUser.getOrgId(), examTask.getFlowId(), examTask.getId(), sysUser.getId(), sysUser.getId(), FlowApproveSetupEnum.END.getTitle(), tfFlowJoin.getObjectTable());
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.END);
            tfFlowLog.setApproveSetup(FlowApproveSetupEnum.END.getSetup());
            tfFlowLogService.save(tfFlowLog);

            //命题任务修改
//            examTask.setFlowId(null);
            examTask.setStatus(ExamStatusEnum.STAGE);
            examTask.updateInfo(sysUser.getId());
            examTaskService.updateById(examTask);
            runtimeService.deleteProcessInstance(flowId, FlowApproveSetupEnum.END.getTitle());
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(SystemConstant.REDIS_LOCK_FLOW_PREFIX + flowId);
        }
    }

    /**
     * 流程审批启动
     *
     * @param map
     */
    protected void flowApproveStart(Map<String, Object> map) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Optional.ofNullable(map.get(SystemConstant.FLOW_ID)).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程id不能为空"));
        String flowId = (String) map.get(SystemConstant.FLOW_ID);
        TFCustomTypeEnum flowType = (TFCustomTypeEnum) map.get(SystemConstant.FLOW_TYPE);

        Optional.ofNullable(map.get(SystemConstant.FLOW_STATUS)).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程状态不能为空"));
        FlowStatusEnum flowStatusEnum = (FlowStatusEnum) map.get(SystemConstant.FLOW_STATUS);

        String approveId = Objects.nonNull(map.get(SystemConstant.APPROVE_ID)) ? String.valueOf((Long) map.get(SystemConstant.APPROVE_ID)) : null;
        tfFlowApproveService.save(new TFFlowApprove(sysUser.getSchoolId(), sysUser.getOrgId(), SystemConstant.convertIdToLong(flowId), sysUser.getId(), flowStatusEnum, sysUser.getId()));

        Long objectId = (Long) map.get(SystemConstant.OBJECT_ID);
//        //TODO 测试用
//        if (flowType == TFCustomTypeEnum.ELECTRON_FLOW) {//如果是电子交卷审核，就查询命题任务
//            ExamTask examTask = examTaskService.getById(objectId);
//            examTask.setFlowId(SystemConstant.convertIdToLong(flowId));
//            examTaskService.updateById(examTask);
//        }
//
//        String id = (String) map.get(SystemConstant.FLOW_ENTITY_ID);
//        TFFlowJoin tfFlowJoin = tfFlowJoinService.findByFlowLevel(objectId);
//        tfFlowJoin.setFlowEntityId(SystemConstant.convertIdToLong(id));
//        tfFlowJoin.setFlowId(SystemConstant.convertIdToLong(flowId));
//        tfFlowJoin.setObjectTable(flowType.getTable());
//        tfFlowJoinService.updateById(tfFlowJoin);

        tfFlowLogService.save(new TFFlowLog(sysUser.getSchoolId(), sysUser.getOrgId(), SystemConstant.convertIdToLong(flowId), objectId, sysUser.getId(), sysUser.getId(), "提交流程", approveId, FlowApproveOperationEnum.SUBMIT, flowType.getTable()));
    }

    /**
     * 审批任务
     *
     * @param map
     */
    @Override
    @Transactional
    public Map<String, Object> taskApprove(Map<String, Object> map) {
        String flowTaskId = null;
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Optional.ofNullable(map.get(SystemConstant.FLOW_TASK_ID)).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程任务id不能为空"));
        flowTaskId = (String) map.get(SystemConstant.FLOW_TASK_ID);
        Task task = taskService.createTaskQuery().taskId(flowTaskId).singleResult();
        Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程任务为空或该流程已被他人审核，请刷新再试！"));
        Long flowId = SystemConstant.convertIdToLong(task.getProcessInstanceId());
        boolean lock = redisUtil.lock(SystemConstant.REDIS_LOCK_FLOW_TASK_PREFIX + flowId, SystemConstant.REDIS_LOCK_FLOW_TASK_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在审批中，请稍候再试！");
        }
        try {
            ExamTask examTask = examTaskService.findByFlowId(flowId);

            //获取当前流程节点
            String currActivityId = task.getTaskDefinitionKey();
            BpmnModel bpmnModel = repositoryService.getBpmnModel(task.getProcessDefinitionId());
            FlowNode currFlow = (FlowNode) bpmnModel.getMainProcess().getFlowElement(currActivityId);

            //流程审批
            TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(flowId, examTask.getSchoolId());
            tfFlowApprove = Objects.isNull(tfFlowApprove) ? new TFFlowApprove(sysUser.getSchoolId(), sysUser.getOrgId(), flowId, sysUser.getId(), FlowStatusEnum.FINISH, sysUser.getId()) : tfFlowApprove;

            Map<String, Object> objectMap = new HashMap<>();
            TFFlowLog tfFlowLog = null;

            TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.findByFlowId(flowId);
            Map<String, Object> flowProcessVarMap = this.getFlowProcessVarMap(tfCustomFlowEntity);

            TFFlowJoin tfFlowJoin = tfFlowJoinService.findByRootLevel(flowId);

            String remark = null;
            FlowTaskResult currFlowTaskResult = null, nextFlowTaskResult = null;
            Map<String, CustomFlowVarDto> agginessMap = (Map<String, CustomFlowVarDto>) flowProcessVarMap.get(SystemConstant.AGGINESS_MAP);
            if (Objects.nonNull(tfFlowApprove.getTransferAssignee())) {
                agginessMap = JacksonUtil.readJson(tfFlowApprove.getTransferAssignee(), Map.class);
            }
            Gson gson = new Gson();
            Integer nrOfCompletedInstances = null, nrOfInstances = null;
            boolean multiInstance = false;
            if (currFlow instanceof UserTask) {
                UserTask userTask = (UserTask) currFlow;
                String id = userTask.getId().substring(userTask.getId().length() - 1, userTask.getId().length());
                if (currFlow.getBehavior() instanceof MultiInstanceActivityBehavior) {
                    nrOfCompletedInstances = (Integer) taskService.getVariable(task.getId(), DefaultInstanceConvertToMultiInstance.NR_OF_COMPLETED_INSTANCES);
                    nrOfCompletedInstances++;
                    nrOfInstances = (Integer) taskService.getVariable(task.getId(), DefaultInstanceConvertToMultiInstance.NR_OF_INSTANCES);
                    multiInstance = true;
                }
                remark = Objects.nonNull(map.get(SystemConstant.APPROVE_REMARK)) ? map.get(SystemConstant.APPROVE_REMARK).toString() : null;
                //流程流水日志
                tfFlowLog = new TFFlowLog(sysUser.getSchoolId(), sysUser.getOrgId(), flowId, examTask.getId(), sysUser.getId(), sysUser.getId(), remark, tfFlowJoin.getObjectTable());
                //判断流程审批是通过还是驳回
                FlowApprovePassEnum approvePass = (FlowApprovePassEnum) map.get(SystemConstant.APPROVE_OPERATION);
                Map<String, FlowTaskResult> setupMap = (Map<String, FlowTaskResult>) flowProcessVarMap.get(SystemConstant.SETUP_MAP);
                //获取当前流程和下一流程setup
                currFlowTaskResult = gson.fromJson(gson.toJson(setupMap.get(userTask.getId())), FlowTaskResult.class);
                if (approvePass == FlowApprovePassEnum.PASS) {
                    int currSetup = currFlowTaskResult.getSetup().intValue();
                    currSetup = currSetup == setupMap.size() - 1 ? 0 : currSetup + 1;
                    //驳回再提交时，可能需要走网关路线
                    //获取驳回路线
                    FlowTaskResult flowTaskResultRejectLinkDto = null;
                    Map<String, FlowTaskResult> rejectLinkMap = null;
                    if (Objects.nonNull(tfFlowApprove.getRejectLink())) {
                        rejectLinkMap = JacksonUtil.readJson(tfFlowApprove.getRejectLink(), Map.class);
                        if (Objects.nonNull(rejectLinkMap.get(userTask.getId()))) {
                            flowTaskResultRejectLinkDto = gson.fromJson(gson.toJson(rejectLinkMap.get(userTask.getId())), FlowTaskResult.class);
                        }
                    }

                    nextFlowTaskResult = this.getNextFlowTaskResult(nrOfCompletedInstances,
                            nrOfInstances,
                            currFlowTaskResult,
                            nextFlowTaskResult,
                            Objects.nonNull(flowTaskResultRejectLinkDto) ? flowTaskResultRejectLinkDto.getSetup() : currSetup,
                            setupMap,
                            gson,
                            approvePass);

                    this.approvePass(currFlowTaskResult,
                            setupMap,
                            multiInstance,
                            nrOfCompletedInstances,
                            nrOfInstances,
                            tfFlowApprove,
                            tfFlowLog,
                            nextFlowTaskResult,
                            map,
                            id);

                    if (Objects.nonNull(flowTaskResultRejectLinkDto)) {
                        rejectLinkMap.remove(userTask.getId());
                        tfFlowApprove.setRejectLink(rejectLinkMap.size() == 0 ? null : JacksonUtil.parseJson(rejectLinkMap));
                    }
                } else if (approvePass == FlowApprovePassEnum.REJECT) {
                    Optional.ofNullable(map.get(SystemConstant.APPROVE_SETUP)).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程驳回节点不能为空"));
                    Integer rejectSetup = Integer.parseInt(map.get(SystemConstant.APPROVE_SETUP).toString());

                    nextFlowTaskResult = this.getNextFlowTaskResult(nrOfCompletedInstances,
                            nrOfInstances,
                            currFlowTaskResult,
                            nextFlowTaskResult,
                            rejectSetup,
                            setupMap,
                            gson,
                            approvePass);

                    this.approveReject(multiInstance,
                            map,
                            id,
                            currFlowTaskResult,
                            nextFlowTaskResult,
                            tfFlowApprove,
                            tfFlowLog,
                            agginessMap,
                            gson);
                }
                //获取流程变量
                this.getAgginess(nextFlowTaskResult,
                        gson,
                        map,
                        agginessMap);
            }
            this.setApproveInfo(tfFlowApprove,
                    tfFlowLog,
                    sysUser,
                    map,
                    currFlow,
                    nrOfCompletedInstances,
                    nrOfInstances,
                    currFlowTaskResult,
                    nextFlowTaskResult,
                    gson,
                    agginessMap,
                    examTask,
                    objectMap);
            //当前实例的执行到哪个节点
//            int i = 1 / 0;
            taskService.complete(String.valueOf(flowTaskId), map);
            return objectMap;
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(SystemConstant.REDIS_LOCK_FLOW_TASK_PREFIX + flowId);
        }
        return map;
    }

    /**
     * 设置审批信息
     *
     * @param tfFlowApprove
     * @param tfFlowLog
     * @param sysUser
     * @param map
     * @param currFlow
     * @param nrOfCompletedInstances
     * @param nrOfInstances
     * @param currFlowTaskResult
     * @param nextFlowTaskResult
     * @param gson
     * @param agginessMap
     * @param examTask
     * @param objectMap
     */
    protected void setApproveInfo(TFFlowApprove tfFlowApprove,
                                  TFFlowLog tfFlowLog,
                                  SysUser sysUser,
                                  Map<String, Object> map,
                                  FlowNode currFlow,
                                  Integer nrOfCompletedInstances,
                                  Integer nrOfInstances,
                                  FlowTaskResult currFlowTaskResult,
                                  FlowTaskResult nextFlowTaskResult,
                                  Gson gson,
                                  Map<String, CustomFlowVarDto> agginessMap,
                                  ExamTask examTask,
                                  Map<String, Object> objectMap) {
        tfFlowApprove.updateInfo(sysUser.getId());
        if (tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.APPROVE) {
            tfFlowApprove.setApproveSetup(nrOfCompletedInstances);
            tfFlowApprove.setApproveMaxSetup(nrOfInstances);
        } else {
            tfFlowApprove.setApproveSetup(null);
            tfFlowApprove.setApproveMaxSetup(null);
        }
        tfFlowApprove.setApproveId(sysUser.getId());
        tfFlowLog.setApproveSetup(currFlowTaskResult.getSetup());
        tfFlowLog.setApproveMultiSetup(nrOfCompletedInstances);
        tfFlowLog.setPaperAttachmentId((String) map.get(SystemConstant.PAPAER_ATTACHMENT_IDS));
        CustomFlowVarDto customFlowVarDto = gson.fromJson(gson.toJson(agginessMap.get(nextFlowTaskResult.getTaskKey())), CustomFlowVarDto.class);
        if (Objects.nonNull(customFlowVarDto)) {
            if (tfFlowLog.getApproveOperation() == FlowApproveOperationEnum.APPROVE && Objects.nonNull(nrOfCompletedInstances)
                    && Objects.nonNull(nrOfInstances)
                    && nrOfCompletedInstances.intValue() != nrOfInstances.intValue()) {
                if (currFlow.getBehavior() instanceof SequentialMultiInstanceBehavior) {//串行
                    tfFlowLog.setPendApproveId(customFlowVarDto.getApproveIds().subList(nrOfCompletedInstances, nrOfInstances).toString().replaceAll("\\[", "").replaceAll("\\]", ""));
                } else if (currFlow.getBehavior() instanceof ParallelMultiInstanceBehavior) {//并行
                    Map<String, CustomFlowVarDto> allAgginessMap = null;
                    CustomFlowVarDto customFlowVarAllDto = null;
                    if (Objects.nonNull(tfFlowApprove.getAllAssignee())) {
                        allAgginessMap = JacksonUtil.readJson(tfFlowApprove.getAllAssignee(), Map.class);
                        customFlowVarAllDto = gson.fromJson(gson.toJson(allAgginessMap.get(nextFlowTaskResult.getTaskKey())), CustomFlowVarDto.class);
                    } else {
                        allAgginessMap = new HashMap<>();
                        customFlowVarAllDto = customFlowVarDto;
                    }
                    allAgginessMap.put(nextFlowTaskResult.getTaskKey(), customFlowVarAllDto);
                    List<String> approveIds = customFlowVarAllDto.getApproveIds();
                    approveIds.remove(String.valueOf(sysUser.getId()));
                    if (Objects.nonNull(approveIds) && approveIds.size() > 0) {
                        tfFlowLog.setPendApproveId(approveIds.toString().replaceAll("\\[", "").replaceAll("\\]", ""));
                    }
                    tfFlowApprove.setAllAssignee(JacksonUtil.parseJson(allAgginessMap));
                }
            } else {
                if (Objects.nonNull(tfFlowApprove.getAllAssignee())) {
                    Map<String, CustomFlowVarDto> allAgginessMap = JacksonUtil.readJson(tfFlowApprove.getAllAssignee(), Map.class);
                    allAgginessMap.remove(currFlowTaskResult.getTaskKey());
                    tfFlowApprove.setAllAssignee(allAgginessMap.size() == 0 ? null : JacksonUtil.parseJson(allAgginessMap));
                }
                tfFlowLog.setPendApproveId(customFlowVarDto.getApproveIds().toString().replaceAll("\\[", "").replaceAll("\\]", ""));
            }
            tfFlowLog.setApproveUserApproveType(customFlowVarDto.getMultipleUserApproveType());
        }
        map.put(SystemConstant.APPROVE_TF_FLOW_LOG, tfFlowLog);
        tfFlowApproveService.saveOrUpdate(tfFlowApprove);
        tfFlowLogService.save(tfFlowLog);
        examTask.updateInfo(sysUser.getId());
        examTaskService.updateById(examTask);
        objectMap.putIfAbsent("tfFlowApprove", tfFlowApprove);
        objectMap.putIfAbsent("examTask", examTask);
        objectMap.putIfAbsent("sysUser", sysUser);
    }

    /**
     * 获取审批人
     *
     * @param nextFlowTaskResult
     * @param gson
     * @param map
     * @param agginessMap
     */
    protected void getAgginess(FlowTaskResult nextFlowTaskResult,
                               Gson gson,
                               Map<String, Object> map,
                               Map<String, CustomFlowVarDto> agginessMap) {
        if (Objects.nonNull(agginessMap.get(nextFlowTaskResult.getTaskKey()))) {
            CustomFlowVarDto customFlowVarDto = gson.fromJson(gson.toJson(agginessMap.get(nextFlowTaskResult.getTaskKey())), CustomFlowVarDto.class);
            map.computeIfAbsent(customFlowVarDto.getFlowTaskVar(), v -> customFlowVarDto.getApproveIds().size() == 1 ? customFlowVarDto.getApproveIds().get(0) : customFlowVarDto.getApproveIds());
        }
    }

    /**
     * 审批驳回
     *
     * @param multiInstance
     * @param map
     * @param id
     * @param currFlowTaskResult
     * @param nextFlowTaskResult
     * @param tfFlowApprove
     * @param tfFlowLog
     * @param agginessMap
     * @param gson
     */
    protected void approveReject(boolean multiInstance,
                                 Map<String, Object> map,
                                 String id,
                                 FlowTaskResult currFlowTaskResult,
                                 FlowTaskResult nextFlowTaskResult,
                                 TFFlowApprove tfFlowApprove,
                                 TFFlowLog tfFlowLog,
                                 Map<String, CustomFlowVarDto> agginessMap,
                                 Gson gson) {
        if (multiInstance) {
            map.computeIfAbsent(FlowApproveOperationEnum.REJECT.getId() + id, v -> 1);
        }
        map.put(SystemConstant.APPROVE, nextFlowTaskResult.getTaskKey());
        tfFlowApprove.setSetup(nextFlowTaskResult.getSetup());
        tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.REJECT);
        CustomFlowVarDto customFlowVarDto = gson.fromJson(gson.toJson(agginessMap.get(currFlowTaskResult.getTaskKey())), CustomFlowVarDto.class);
        if (Objects.nonNull(customFlowVarDto) && customFlowVarDto.getRejectResubmitType() == CustomFlowRejectResubmitTypeEnum.PREV_STEP) {
            Map<String, FlowTaskResult> rejectLinkMap = null;
            if (Objects.nonNull(tfFlowApprove.getRejectLink())) {
                rejectLinkMap = JacksonUtil.readJson(tfFlowApprove.getRejectLink(), Map.class);
            } else {
                rejectLinkMap = new HashMap<>();
            }
            rejectLinkMap.put(nextFlowTaskResult.getTaskKey(), currFlowTaskResult);
            tfFlowApprove.setRejectLink(JacksonUtil.parseJson(rejectLinkMap));
        }
    }

    /**
     * 获取下级审核节点
     *
     * @param nrOfCompletedInstances
     * @param nrOfInstances
     * @param currFlowTaskResult
     * @param nextFlowTaskResult
     * @param currSetup
     * @param setupMap
     * @param gson
     * @param approvePass
     * @return
     */
    protected FlowTaskResult getNextFlowTaskResult(Integer nrOfCompletedInstances,
                                                   Integer nrOfInstances,
                                                   FlowTaskResult currFlowTaskResult,
                                                   FlowTaskResult nextFlowTaskResult,
                                                   int currSetup,
                                                   Map<String, FlowTaskResult> setupMap,
                                                   Gson gson,
                                                   FlowApprovePassEnum approvePass) {
        if (approvePass == FlowApprovePassEnum.PASS) {
            if (Objects.nonNull(nrOfCompletedInstances)
                    && Objects.nonNull(nrOfInstances)
                    && nrOfCompletedInstances.intValue() != nrOfInstances.intValue()) {
                nextFlowTaskResult = currFlowTaskResult;
            } else {
                nextFlowTaskResult = this.getNextFlowTaskResult(setupMap, gson, currSetup, nextFlowTaskResult);
            }
        } else if (approvePass == FlowApprovePassEnum.REJECT) {
            nextFlowTaskResult = this.getNextFlowTaskResult(setupMap, gson, currSetup, nextFlowTaskResult);
        }
        Optional.ofNullable(nextFlowTaskResult).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("该节点不能审批通过"));
        return nextFlowTaskResult;
    }

    /**
     * 获取下级审核节点
     *
     * @param setupMap
     * @param gson
     * @param currSetup
     * @param nextFlowTaskResult
     * @return
     */
    protected FlowTaskResult getNextFlowTaskResult(Map<String, FlowTaskResult> setupMap,
                                                   Gson gson,
                                                   int currSetup,
                                                   FlowTaskResult nextFlowTaskResult) {
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResultTemp = gson.fromJson(gson.toJson(entry.getValue()), FlowTaskResult.class);
            if (flowTaskResultTemp.getSetup().intValue() == currSetup) {
                nextFlowTaskResult = flowTaskResultTemp;
                break;
            }
        }
        return nextFlowTaskResult;
    }

    /**
     * 审批通过
     *
     * @param currFlowTaskResult
     * @param setupMap
     * @param multiInstance
     * @param nrOfCompletedInstances
     * @param nrOfInstances
     * @param tfFlowApprove
     * @param tfFlowLog
     * @param nextFlowTaskResult
     * @param map
     * @param id
     */
    protected void approvePass(FlowTaskResult currFlowTaskResult,
                               Map<String, FlowTaskResult> setupMap,
                               boolean multiInstance,
                               Integer nrOfCompletedInstances,
                               Integer nrOfInstances,
                               TFFlowApprove tfFlowApprove,
                               TFFlowLog tfFlowLog,
                               FlowTaskResult nextFlowTaskResult,
                               Map<String, Object> map,
                               String id) {
        if (multiInstance) {
            map.computeIfAbsent(FlowApproveOperationEnum.REJECT.getId() + id, v -> 0);
        }
        if (currFlowTaskResult.getSetup().intValue() == setupMap.size() - 1) {
            if (multiInstance && Objects.nonNull(nrOfCompletedInstances)
                    && Objects.nonNull(nrOfInstances)
                    && nrOfCompletedInstances.intValue() == nrOfInstances.intValue()) {
                tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
                tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
            } else if (!multiInstance) {
                tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
                tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
            } else {
                tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
                tfFlowApprove.setSetup(nextFlowTaskResult.getSetup());
            }
        } else {
            tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
            tfFlowApprove.setSetup(nextFlowTaskResult.getSetup());
        }
        if (tfFlowApprove.getStatus() == FlowStatusEnum.START) {
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.SUBMIT);
        } else {
            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
        }
        map.put(SystemConstant.APPROVE, nextFlowTaskResult.getTaskKey());
    }

    /**
     * 获取所有流程节点
     *
     * @param flowId
     * @return
     */
    @Override
    public List<FlowTaskResult> getTaskAll(String flowId) {
        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.findByFlowId(SystemConstant.convertIdToLong(flowId));
        Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("自定义流程实体数据为空"));
        return this.getFlowTaskResult(tfCustomFlowEntity);
    }

    /**
     * 获取流程节点信息
     *
     * @param o
     * @return
     */
    protected List<FlowTaskResult> getFlowTaskResult(Object o) {
        Long flowId = null;
        if (o instanceof TFCustomFlow) {
            o = (TFCustomFlow) o;
        } else if (o instanceof TFCustomFlowEntity) {
            o = (TFCustomFlowEntity) o;
            flowId = ((TFCustomFlowEntity) o).getFlowId();
        }
        Map<String, Object> flowProcessVarMap = this.getFlowProcessVarMap(o);
        Map<String, FlowTaskResult> setupMap = (Map<String, FlowTaskResult>) flowProcessVarMap.get(SystemConstant.SETUP_MAP);
        Optional.ofNullable(setupMap).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有步骤数据！"));

        List<FlowTaskResult> flowTaskResultList = new ArrayList<>(setupMap.size());
        Gson gson = new Gson();
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResult = gson.fromJson(gson.toJson(entry.getValue()), FlowTaskResult.class);
            if (flowTaskResult.getSetup() > 0) {
                flowTaskResult.setFlowId(flowId);
                flowTaskResultList.add(flowTaskResult);
            }
        }
        return flowTaskResultList;
    }

    /**
     * 流程删除
     *
     * @param id
     * @return
     */
    @Override
    public Boolean flowDelete(Long id) {
//        TFFlow tfFlow = tfFlowService.getById(id);
//        List<ProcessInstance> processInstanceList = runtimeService.createProcessInstanceQuery().processDefinitionId(tfFlow.getActFlowId()).processDefinitionKey(tfFlow.getFlowKey()).list();
//        if (Objects.nonNull(processInstanceList) && processInstanceList.size() > 0) {
//            throw ExceptionResultEnum.ERROR.exception("有在途的流程，无法删除！");
//        } else {
//            UpdateWrapper<TFFlow> tfFlowUpdateWrapper = new UpdateWrapper<>();
//            tfFlowUpdateWrapper.lambda().eq(TFFlow::getId, id)
//                    .set(TFFlow::getEnable, false);
//            tfFlowService.update(tfFlowUpdateWrapper);
//        }
        return true;
    }

    /**
     * 获取所有流程节点待审批人
     *
     * @param courseCode
     * @param taskId
     * @return
     */
    @Override
    public FlowTaskApprovePeopleAllResult taskApproverPeopleAll(String courseCode, String taskId) {
        FlowTaskApprovePeopleAllResult flowTaskApprovePeopleAllResult = null;
//        List<FlowTaskApprovePeopleResult> flowTaskApprovePeopleResultList = null;
//        if (Objects.nonNull(taskId) && !Objects.equals(taskId, "")) {//说明从已审核的节点开始捞人
//            Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
//            Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程任务为空或该流程已被他人审核，请刷新再试！"));
//            flowTaskApprovePeopleAllResult = new FlowTaskApprovePeopleAllResult(SystemConstant.convertIdToLong(task.getProcessInstanceId()));
//            flowTaskApprovePeopleResultList = new LinkedList();
//            //获取当前流程节点
//            TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId(), Long.parseLong((String) ServletUtil.getRequestHeaderSchoolId()));
//            Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到流程状态记录"));
//            if (tfFlowApprove.getSetup() <= 0) {
//                throw ExceptionResultEnum.ERROR.exception("流程已结束或已终止");
//            }
//            String processDefinitionId = historyService.createHistoricProcessInstanceQuery().processInstanceId(task.getProcessInstanceId()).singleResult().getProcessDefinitionId();
//            ProcessDefinitionEntity processDefinitionEntity = (ProcessDefinitionEntity) ((RepositoryServiceImpl) repositoryService).getDeployedProcessDefinition(processDefinitionId);
//
//            ExamTask examTask = examTaskService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId());
//
//            FlowApproveResult flowApproveResult = tfFlowService.flowApproveUserList(SystemConstant.getHeadOrUserSchoolId(), flowTaskApprovePeopleAllResult.getFlowId());
//            if (tfFlowApprove.getSetup() != FlowApproveSetupEnum.SUBMIT.getSetup()) {
//                Optional.ofNullable(flowApproveResult).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到流程审批记录"));
//            }
//            List<FlowTaskResult> taskTaskAll = getTaskAll(task.getProcessInstanceId());
//            switch (tfFlowApprove.getSetup()) {
//                case -1:
//                    if (Objects.nonNull(flowApproveResult.getStartId())) {
//                        //获取命题已审
//                        List<ApproveUserResult> approveUserMtResult = sysUserService.findById(flowApproveResult.getStartId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SUBMIT.getSetup(), true, approveUserMtResult));
//                    }
//                    if (Objects.nonNull(flowApproveResult.getPrimaryApproveId())) {
//                        //获取主任已审
//                        List<ApproveUserResult> approveUserDirectorList = sysUserService.findById(flowApproveResult.getPrimaryApproveId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), true, approveUserDirectorList));
//                    }
//                    if (Objects.nonNull(flowApproveResult.getSecondApproveId())) {
//                        //获取院长已审
//                        List<ApproveUserResult> approveUserPresidentList = sysUserService.findById(flowApproveResult.getSecondApproveId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), true, approveUserPresidentList));
//                    }
//                    if (Objects.nonNull(taskTaskAll) && taskTaskAll.size() > 3) {
//                        if (Objects.nonNull(flowApproveResult.getThreeApproveId())) {
//                            //获取命题已审
//                            List<ApproveUserResult> approveUserTeacherResult = sysUserService.findById(flowApproveResult.getThreeApproveId());
//                            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), true, approveUserTeacherResult));
//                        }
//                        if (Objects.nonNull(flowApproveResult.getFourApproveId())) {
//                            //获取印刷员已审
//                            List<ApproveUserResult> approveUserPrintResult = sysUserService.findById(flowApproveResult.getFourApproveId());
//                            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), true, approveUserPrintResult));
//                        }
//                    }
//                    break;
//                case 0:
//                    //广东医科大学流程
//                    if (Objects.nonNull(processDefinitionEntity) && (processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)
//                            || processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_SUB_FLOW_KEY))) {
//                        //获取命题已审
//                        List<ApproveUserResult> approveUserMtResult = sysUserService.findById(flowApproveResult.getStartId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SUBMIT.getSetup(), true, approveUserMtResult));
//                        //获取主任已审
//                        List<ApproveUserResult> approveUserDirectorList = sysUserService.findById(flowApproveResult.getPrimaryApproveId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), true, approveUserDirectorList));
//                        //获取院长已审
//                        List<ApproveUserResult> approveUserPresidentList = sysUserService.findById(flowApproveResult.getSecondApproveId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), true, approveUserPresidentList));
//
//                        if (Objects.nonNull(taskTaskAll) && taskTaskAll.size() > 3) {
//                            if (Objects.nonNull(flowApproveResult.getThreeApproveId())) {
//                                //获取命题已审
//                                List<ApproveUserResult> approveUserTeacherResult = sysUserService.findById(flowApproveResult.getThreeApproveId());
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), true, approveUserTeacherResult));
//                            }
//                            if (Objects.nonNull(flowApproveResult.getFourApproveId())) {
//                                //获取印刷员已审
//                                List<ApproveUserResult> approveUserPrintResult = sysUserService.findById(flowApproveResult.getFourApproveId());
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), true, approveUserPrintResult));
//                            }
//                        }
//                    }//江西中医药大学
//                    else if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.JXZYY_FLOW_KEY)) {
//                        //获取命题已审
//                        List<ApproveUserResult> approveUserMtResult = sysUserService.findById(flowApproveResult.getStartId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SUBMIT.getSetup(), true, approveUserMtResult));
//                        //获取主任已审
//                        List<ApproveUserResult> approveUserDirectorList = sysUserService.findById(flowApproveResult.getPrimaryApproveId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), true, approveUserDirectorList));
//                    } else {
//                        throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//                    }
//                    break;
//                case 1:
//                    if (tfFlowApprove.getModel() == FlowModelEnum.SYSTEM) {
//                        flowTaskApprovePeopleResultList = getApproveUserAll(examTask.getCourseCode(), flowTaskApprovePeopleResultList, Objects.nonNull(processDefinitionEntity) ? processDefinitionEntity.getKey() : null, task.getProcessInstanceId(), taskTaskAll);
//                    } else {
//                        //广东医科大学流程
//                        if (Objects.nonNull(processDefinitionEntity) && (processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)
//                                || processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_SUB_FLOW_KEY))) {
//                            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), false, new ArrayList<>()));
//                            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), false, new ArrayList<>()));
//                            if (Objects.nonNull(taskTaskAll) && taskTaskAll.size() > 3) {
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), false, new ArrayList<>()));
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), false, new ArrayList<>()));
//                            }
//                        }//江西中医药大学
//                        else if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.JXZYY_FLOW_KEY)) {
//                            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), false, new ArrayList<>()));
//                        } else {
//                            throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//                        }
//                    }
//                    break;
//                case 2:
//                    //广东医科大学流程
//                    if (Objects.nonNull(processDefinitionEntity) && (processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)
//                            || processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_SUB_FLOW_KEY))) {
//                        //获取命题已审
//                        List<ApproveUserResult> approveUserMtResult = sysUserService.findById(flowApproveResult.getStartId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SUBMIT.getSetup(), true, approveUserMtResult));
//
//                        if (tfFlowApprove.getModel() == FlowModelEnum.SYSTEM) {
//                            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
//                            if (Objects.nonNull(sysUser.getOrgId())) {
//                                //获取院长审批人
//                                List<ApproveUserResult> sysUserPresidentList = sysUserService.findByPresidentApprove(new ArrayList(Arrays.asList(sysUser.getOrgId())), null);
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), false, sysUserPresidentList));
//                            }
//                            if (Objects.nonNull(taskTaskAll) && taskTaskAll.size() > 3) {
//                                //获取命题审批人
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), false, approveUserMtResult));
//                                //获取印刷员审批人
//                                List<ApproveUserResult> sysUserPrintList = this.findByPrintApprove(SystemConstant.convertIdToLong(task.getProcessInstanceId()));
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), false, sysUserPrintList));
//                            }
//                        } else {
//                            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), false, new ArrayList<>()));
//                            if (Objects.nonNull(taskTaskAll) && taskTaskAll.size() > 3) {
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), false, new ArrayList<>()));
//                                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), false, new ArrayList<>()));
//                            }
//                        }
//                    }//江西中医药大学
//                    else if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.JXZYY_FLOW_KEY)) {
//                        //获取命题已审
//                        List<ApproveUserResult> approveUserMtResult = sysUserService.findById(flowApproveResult.getStartId());
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SUBMIT.getSetup(), true, approveUserMtResult));
//                    } else {
//                        throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//                    }
//                    break;
//                case 3:
//                    //获取命题已审
//                    List<ApproveUserResult> approveUserMtResult = sysUserService.findById(flowApproveResult.getStartId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SUBMIT.getSetup(), true, approveUserMtResult));
//                    //获取主任已审
//                    List<ApproveUserResult> sysUserDirectorList = sysUserService.findById(flowApproveResult.getPrimaryApproveId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), true, sysUserDirectorList));
//                    if (Objects.nonNull(taskTaskAll) && taskTaskAll.size() > 3) {
//                        //获取命题审批人
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), false, approveUserMtResult));
//                        //获取印刷员审批人
//                        List<ApproveUserResult> sysUserPrintList = this.findByPrintApprove(SystemConstant.convertIdToLong(task.getProcessInstanceId()));
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), false, sysUserPrintList));
//                    }
//                    break;
//                case 4:
//                    //获取命题已审
//                    approveUserMtResult = sysUserService.findById(flowApproveResult.getStartId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SUBMIT.getSetup(), true, approveUserMtResult));
//                    //获取主任已审
//                    sysUserDirectorList = sysUserService.findById(flowApproveResult.getPrimaryApproveId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), true, sysUserDirectorList));
//                    //获取院长已审
//                    List<ApproveUserResult> sysUserPresidentList = sysUserService.findById(flowApproveResult.getSecondApproveId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), true, sysUserPresidentList));
//                    //获取印刷员审批人
//                    List<ApproveUserResult> sysUserPrintList = this.findByPrintApprove(SystemConstant.convertIdToLong(task.getProcessInstanceId()));
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), false, sysUserPrintList));
//                    break;
//                case 5:
//                    //获取命题已审
//                    approveUserMtResult = sysUserService.findById(flowApproveResult.getStartId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SUBMIT.getSetup(), true, approveUserMtResult));
//                    //获取主任已审
//                    sysUserDirectorList = sysUserService.findById(flowApproveResult.getPrimaryApproveId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), true, sysUserDirectorList));
//                    //获取院长已审
//                    sysUserPresidentList = sysUserService.findById(flowApproveResult.getSecondApproveId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), true, sysUserPresidentList));
//                    //获取命题审核已审
//                    List<ApproveUserResult> approveUserTeacherResult = sysUserService.findById(flowApproveResult.getThreeApproveId());
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), true, approveUserTeacherResult));
//                    break;
//                default:
//                    break;
//            }
//        } else {//获取整个节点审批人
//            Map<String, List<String>> flowSchoolCodeMap = SystemConstant.getFlowSchoolCodeMap();
//            BasicSchool basicSchool = (BasicSchool) ServletUtil.getRequestSchool();
//            if (flowSchoolCodeMap.get(SystemConstant.GDYKDX_FLOW_KEY).contains(basicSchool.getCode())) {
//                if (Objects.isNull(courseCode) || Objects.equals(courseCode, "")) {
//                    throw ExceptionResultEnum.ERROR.exception("课程编码为空");
//                }
//                flowTaskApprovePeopleResultList = getApproveUserAll(courseCode, flowTaskApprovePeopleResultList, SystemConstant.GDYKDX_FLOW_KEY, null, null);
//                flowTaskApprovePeopleAllResult = new FlowTaskApprovePeopleAllResult();
//            } else if (flowSchoolCodeMap.get(SystemConstant.JXZYY_FLOW_KEY).contains(basicSchool.getCode())) {//江西中医药大学
//                flowTaskApprovePeopleResultList = getApproveUserAll(courseCode, flowTaskApprovePeopleResultList, SystemConstant.JXZYY_FLOW_KEY, null, null);
//                flowTaskApprovePeopleAllResult = new FlowTaskApprovePeopleAllResult();
//            } else {
//                throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//            }
//        }
//        flowTaskApprovePeopleAllResult.setApproveUserList(flowTaskApprovePeopleResultList);
        return flowTaskApprovePeopleAllResult;
    }

    /**
     * 流程节点转他人审批
     *
     * @param userId
     * @param taskId
     * @return
     */
    @Override
    @Transactional
    public boolean taskApproverExchange(String userId, String taskId) {
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
        Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程任务为空或该流程已被他人审核，请刷新再试！"));
        String flowId = task.getProcessInstanceId();
        TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(SystemConstant.convertIdToLong(flowId), Long.parseLong((String) ServletUtil.getRequestHeaderSchoolId()));
        Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到流程状态记录"));
        if (tfFlowApprove.getSetup() <= 0) {
            throw ExceptionResultEnum.ERROR.exception("流程已结束或已终止");
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.nonNull(task) && Objects.nonNull(task.getAssignee()) && Objects.equals(task.getAssignee(), userId)) {
            throw ExceptionResultEnum.ERROR.exception("不能转给自己");
        }
        Task taskUser = taskService.createTaskQuery().taskId(taskId).taskAssignee(userId).singleResult();
        Task taskCandidateUser = taskService.createTaskQuery().taskId(taskId).taskCandidateUser(userId).singleResult();
        if (Objects.nonNull(taskUser) || Objects.nonNull(taskCandidateUser)) {
            throw ExceptionResultEnum.ERROR.exception("该审批人已有审批权限，无需流转");
        }

        //获取当前流程节点
        String currActivityId = task.getTaskDefinitionKey();
        BpmnModel bpmnModel = repositoryService.getBpmnModel(task.getProcessDefinitionId());
        FlowNode currFlow = (FlowNode) bpmnModel.getMainProcess().getFlowElement(currActivityId);
        UserTask userTask = null;
        if (currFlow instanceof UserTask) {
            userTask = (UserTask) currFlow;
        }
        Integer nrOfCompletedInstances = null, nrOfInstances = null;
        boolean multiInstance = false;
        if (currFlow.getBehavior() instanceof MultiInstanceActivityBehavior) {
            nrOfCompletedInstances = (Integer) taskService.getVariable(task.getId(), DefaultInstanceConvertToMultiInstance.NR_OF_COMPLETED_INSTANCES);
            nrOfInstances = (Integer) taskService.getVariable(task.getId(), DefaultInstanceConvertToMultiInstance.NR_OF_INSTANCES);
            multiInstance = true;
        }

        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.findByFlowId(tfFlowApprove.getFlowId());
        Map<String, Object> flowProcessVarMap = this.getFlowProcessVarMap(tfCustomFlowEntity);
        Map<String, CustomFlowVarDto> agginessMap = (Map<String, CustomFlowVarDto>) flowProcessVarMap.get(SystemConstant.AGGINESS_MAP);
        if (Objects.nonNull(tfFlowApprove.getTransferAssignee())) {
            agginessMap = JacksonUtil.readJson(tfFlowApprove.getTransferAssignee(), Map.class);
        }

        ExamTask examTask = examTaskService.findByFlowId(SystemConstant.convertIdToLong(flowId));
        SysUser exchangeUser = sysUserService.getById(userId);

        TFFlowJoin tfFlowJoin = tfFlowJoinService.findByRootLevel(SystemConstant.convertIdToLong(flowId));

        //流程流水日志
        TFFlowLog tfFlowLog = new TFFlowLog(sysUser.getSchoolId(), sysUser.getOrgId(), examTask.getFlowId(), examTask.getId(), sysUser.getId(), sysUser.getRealName() + "转" + exchangeUser.getRealName() + "审批", FlowApproveOperationEnum.EXCHANGE, sysUser.getId(), String.valueOf(exchangeUser.getId()), tfFlowJoin.getObjectTable());
        Gson gson = new Gson();
        CustomFlowVarDto customFlowVarDto = gson.fromJson(gson.toJson(agginessMap.get(userTask.getId())), CustomFlowVarDto.class);
        if (customFlowVarDto.getApproveIds().size() == 1) {
            customFlowVarDto.setApproveIds(new ArrayList<>(Arrays.asList(userId)));
        } else {
            if (multiInstance && Objects.nonNull(nrOfCompletedInstances)) {
                if (currFlow.getBehavior() instanceof SequentialMultiInstanceBehavior) {//串行
                    List<String> approveIds = customFlowVarDto.getApproveIds();
                    approveIds.remove(nrOfCompletedInstances.intValue());
                    approveIds.add(nrOfCompletedInstances.intValue(), userId);
                    tfFlowLog.setPendApproveId(approveIds.subList(nrOfCompletedInstances, nrOfInstances).toString().replaceAll("\\[", "").replaceAll("\\]", ""));
                } else if (currFlow.getBehavior() instanceof ParallelMultiInstanceBehavior) {//并行
                    removeApproveUserId(customFlowVarDto, String.valueOf(sysUser.getId()), userId);
                    if (Objects.nonNull(tfFlowApprove.getAllAssignee())) {
                        Map<String, CustomFlowVarDto> allAgginessMap = JacksonUtil.readJson(tfFlowApprove.getAllAssignee(), Map.class);
                        if (Objects.nonNull(allAgginessMap.get(userTask.getId()))) {
                            CustomFlowVarDto customFlowVarAllDto = gson.fromJson(gson.toJson(allAgginessMap.get(userTask.getId())), CustomFlowVarDto.class);
                            List<String> approveIds = customFlowVarAllDto.getApproveIds();
                            approveIds.remove(String.valueOf(sysUser.getId()));
                            approveIds.add(userId);
                            allAgginessMap.put(userTask.getId(), customFlowVarAllDto);
                            tfFlowLog.setPendApproveId(approveIds.toString().replaceAll("\\[", "").replaceAll("\\]", ""));
                        }
                        tfFlowApprove.setAllAssignee(JacksonUtil.parseJson(allAgginessMap));
                    } else {
                        tfFlowLog.setPendApproveId(customFlowVarDto.getApproveIds().subList(nrOfCompletedInstances, nrOfInstances).toString().replaceAll("\\[", "").replaceAll("\\]", ""));
                    }
                }
            } else {
                removeApproveUserId(customFlowVarDto, String.valueOf(sysUser.getId()), userId);
            }
        }
        ApproveUserResult approveUserResult = sysUserService.findByIds(customFlowVarDto.getApproveIds().stream().map(x -> SystemConstant.convertIdToLong(x)).collect(Collectors.toList()));
        customFlowVarDto.setApproveUserNames(approveUserResult.getPendApproveUsers());
        agginessMap.put(userTask.getId(), customFlowVarDto);
        tfFlowApprove.setTransferAssignee(JacksonUtil.parseJson(agginessMap));
        tfFlowApproveService.updateById(tfFlowApprove);

        tfFlowLog.setApproveMultiSetup(Objects.nonNull(nrOfCompletedInstances) ? nrOfCompletedInstances++ : nrOfCompletedInstances);
        tfFlowLog.setApproveSetup(tfFlowApprove.getSetup());
        tfFlowLog.setApproveUserApproveType(customFlowVarDto.getMultipleUserApproveType());
        tfFlowLogService.save(tfFlowLog);

//        int i = 1 / 0;
        taskService.setAssignee(taskId, userId);
        return true;
    }

    /**
     * 删除审批人
     *
     * @param customFlowVarDto
     * @param userId
     * @param exchangeUserId
     */
    protected void removeApproveUserId(CustomFlowVarDto customFlowVarDto, String userId, String exchangeUserId) {
        List<String> approveIds = customFlowVarDto.getApproveIds();
        approveIds.remove(userId);
        approveIds.add(exchangeUserId);
    }

    /**
     * 获取转他人审批人
     *
     * @param taskId
     * @param realName
     * @return
     */
    @Override
    public FlowTaskApprovePeopleAllResult taskApproverExchangePeople(String taskId, String realName) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
        Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程任务为空或该流程已被他人审核，请刷新再试！"));
        FlowTaskApprovePeopleAllResult flowTaskApprovePeopleAllResult = new FlowTaskApprovePeopleAllResult(SystemConstant.convertIdToLong(task.getProcessInstanceId()));
//        //获取当前流程节点
//        TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId(), sysUser.getSchoolId());
//        Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到流程状态记录"));
//        if (tfFlowApprove.getSetup() <= 0) {
//            throw ExceptionResultEnum.ERROR.exception("流程已结束或已终止");
//        }
//        String processDefinitionId = historyService.createHistoricProcessInstanceQuery().processInstanceId(task.getProcessInstanceId()).singleResult().getProcessDefinitionId();
//        ProcessDefinitionEntity processDefinitionEntity = (ProcessDefinitionEntity) ((RepositoryServiceImpl) repositoryService).getDeployedProcessDefinition(processDefinitionId);
//
//        ExamTask examTask = null;
//        List<FlowTaskApprovePeopleResult> flowTaskApprovePeopleResultList = new ArrayList<>();
//        switch (tfFlowApprove.getSetup()) {
//            case 1:
//                List<ApproveUserResult> approveUserResultList = null;
//                //广东医科大学流程
//                if (Objects.nonNull(processDefinitionEntity) && (processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)
//                        || processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_SUB_FLOW_KEY))) {
//                    //获取命题审批人
//                    examTask = examTaskService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId());
//                    approveUserResultList = sysUserService.findByAssignApprove(examTask.getCourseCode(), realName);
//                }//江西中医药大学
//                else if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.JXZYY_FLOW_KEY)) {
//                    approveUserResultList = sysUserService.findByJxzyyAssignApprove(realName);
//                } else {
//                    throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//                }
//                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(approveUserResultList));
//                break;
//            case 2:
//                examTask = examTaskService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId());
//                List<ApproveUserResult> sysUserDirectorList = null;
//                //广东医科大学流程
//                if (Objects.nonNull(processDefinitionEntity) && (processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)
//                        || processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_SUB_FLOW_KEY))) {
//                    if (tfFlowApprove.getModel() == FlowModelEnum.SYSTEM) {
//                        //获取教研室主任审批人
//                        sysUserDirectorList = sysUserService.findByDirectorApprove(examTask.getCourseCode(), realName);
//                    } else {
//                        //获取教研室主任审批人
//                        List<SysOrg> sysOrgList = sysOrgService.findByConnectByParentId(examTask.getTeachingRoomId(), true, true);
//                        sysUserDirectorList = sysUserService.findByRootApprove(null, sysOrgList.stream().map(s -> s.getId()).collect(Collectors.toList()), null, FlowApproveNameEnum.DIRECTOR);
//                    }
//                }//江西中医药大学
//                else if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.JXZYY_FLOW_KEY)) {
//                    //获取教研室主任审批人
//                    sysUserDirectorList = sysUserService.findByJxzyyDirectorApprove(realName, FlowApproveNameEnum.DIRECTOR);
//                } else {
//                    throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//                }
//                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(sysUserDirectorList));
//                break;
//            case 3:
//                List<ApproveUserResult> sysUserPresidentList = null;
//                if (tfFlowApprove.getModel() == FlowModelEnum.SYSTEM) {
//                    //获取院长审批人
//                    if (Objects.nonNull(sysUser.getOrgId())) {
//                        sysUserPresidentList = sysUserService.findByPresidentApprove(new ArrayList(Arrays.asList(sysUser.getOrgId())), realName);
//                    }
//                } else {
//                    //获取院长审批人
//                    examTask = examTaskService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId());
//                    List<SysOrg> sysOrgList = sysOrgService.findByConnectByParentId(examTask.getTeachingRoomId(), true, true);
//                    sysUserPresidentList = sysUserService.findByRootApprove(null, sysOrgList.stream().map(s -> s.getId()).collect(Collectors.toList()), null, FlowApproveNameEnum.PRESIDENT);
//                }
//                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(sysUserPresidentList));
//                break;
//            case 4:
//                //获取命题审批人
//                examTask = examTaskService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId());
//                approveUserResultList = sysUserService.findByAssignApprove(examTask.getCourseCode(), realName);
//                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(approveUserResultList));
//                break;
//            case 5:
//                //获取印刷员审批人
//                List<ApproveUserResult> approvePrintUserResultList = this.findByPrintApprove(SystemConstant.convertIdToLong(task.getProcessInstanceId()));
//                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(approvePrintUserResultList));
//                break;
//            default:
//                break;
//        }
//        flowTaskApprovePeopleAllResult.setApproveUserList(flowTaskApprovePeopleResultList);
        return flowTaskApprovePeopleAllResult;
    }

    /**
     * 获取下级节点审批人
     *
     * @param courseCode
     * @param taskId
     * @return
     */
    @Override
    public FlowTaskApprovePeopleAllResult taskApproverNextPeople(String courseCode, String taskId) {
        FlowTaskApprovePeopleAllResult flowTaskApprovePeopleAllResult = null;
//        List<FlowTaskApprovePeopleResult> flowTaskApprovePeopleResultList = null;
//        if (Objects.nonNull(taskId) && !Objects.equals(taskId, "")) {//说明从已审核的节点开始捞人
//            Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
//            Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程任务为空或该流程已被他人审核，请刷新再试！"));
//
//            flowTaskApprovePeopleAllResult = new FlowTaskApprovePeopleAllResult(SystemConstant.convertIdToLong(task.getProcessInstanceId()));
//            //获取当前流程节点
//            TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId(), Long.parseLong((String) ServletUtil.getRequestHeaderSchoolId()));
//            Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到流程状态记录"));
//            if (tfFlowApprove.getSetup() <= 0) {
//                throw ExceptionResultEnum.ERROR.exception("流程已结束或已终止");
//            }
//            List<ApproveUserResult> approveUserResultList = null;
//            flowTaskApprovePeopleResultList = new LinkedList<>();
//
//            String processDefinitionId = historyService.createHistoricProcessInstanceQuery().processInstanceId(task.getProcessInstanceId()).singleResult().getProcessDefinitionId();
//            ProcessDefinitionEntity processDefinitionEntity = (ProcessDefinitionEntity) ((RepositoryServiceImpl) repositoryService).getDeployedProcessDefinition(processDefinitionId);
//
//            ExamTask examTask = examTaskService.findByFlowId(flowTaskApprovePeopleAllResult.getFlowId());
//            List<SysOrg> sysOrgList = sysOrgService.findByConnectByParentId(examTask.getTeachingRoomId(), true, true);
//            switch (tfFlowApprove.getSetup()) {
//                case 1:
//                    //广东医科大学流程
//                    if (Objects.nonNull(processDefinitionEntity) && (processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)
//                            || processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_SUB_FLOW_KEY))) {
//                        //获取教研室主任审批人
//                        approveUserResultList = sysUserService.findByRootApprove(null, sysOrgList.stream().map(s -> s.getId()).collect(Collectors.toList()), null, FlowApproveNameEnum.DIRECTOR);
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), false, approveUserResultList));
//                    }//江西中医药大学
//                    else if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.JXZYY_FLOW_KEY)) {
//                        //获取教研室主任审批人
//                        approveUserResultList = sysUserService.findByJxzyyDirectorApprove(null, FlowApproveNameEnum.DIRECTOR);
//                        flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), false, approveUserResultList));
//                    } else {
//                        throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//                    }
//                    break;
//                case 2:
//                    //获取院长审批人
//                    approveUserResultList = sysUserService.findByRootApprove(null, sysOrgList.stream().map(s -> s.getId()).collect(Collectors.toList()), null, FlowApproveNameEnum.PRESIDENT);
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), false, approveUserResultList));
//                    break;
//                case 3:
//                    //获取命题审批人
//                    approveUserResultList = sysUserService.findByAssignApprove(examTask.getCourseCode(), null);
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), false, approveUserResultList));
//                    break;
//                case 4:
//                    //获取印刷员审批人
//                    approveUserResultList = this.findByPrintApprove(SystemConstant.convertIdToLong(task.getProcessInstanceId()));
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), false, approveUserResultList));
//                    break;
//                default:
//                    break;
//            }
//        } else {//获取整个节点审批人
//            Map<String, List<String>> flowSchoolCodeMap = SystemConstant.getFlowSchoolCodeMap();
//            List<ApproveUserResult> approveUserResultList = null;
//            BasicSchool basicSchool = (BasicSchool) ServletUtil.getRequestSchool();
//            if (flowSchoolCodeMap.get(SystemConstant.GDYKDX_FLOW_KEY).contains(basicSchool.getCode())) {
//                if (Objects.isNull(courseCode) || Objects.equals(courseCode, "")) {
//                    throw ExceptionResultEnum.ERROR.exception("课程编码为空");
//                }
//                approveUserResultList = sysUserService.findByRootApprove(courseCode, null, null, FlowApproveNameEnum.DIRECTOR);
//            } else if (flowSchoolCodeMap.get(SystemConstant.JXZYY_FLOW_KEY).contains(basicSchool.getCode())) {//江西中医药大学
//                approveUserResultList = sysUserService.findByJxzyyDirectorApprove(null, FlowApproveNameEnum.DIRECTOR);
//            } else {
//                throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//            }
//            flowTaskApprovePeopleResultList = new LinkedList<>();
//            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), false, approveUserResultList));
//            flowTaskApprovePeopleAllResult = new FlowTaskApprovePeopleAllResult();
//        }
//        flowTaskApprovePeopleAllResult.setApproveUserList(flowTaskApprovePeopleResultList);
        return flowTaskApprovePeopleAllResult;
    }

    /**
     * 获取印刷员审批人
     *
     * @param flowId
     * @return
     */
    @Override
    public List<ApproveUserResult> findByPrintApprove(Long flowId) {
        ExamTask examTask = examTaskService.findByFlowId(flowId);
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getSchoolId, examTask.getSchoolId())
                .eq(ExamDetailCourse::getCourseCode, examTask.getCourseCode())
                .eq(ExamDetailCourse::getPaperNumber, examTask.getPaperNumber());
        List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.list(examDetailCourseQueryWrapper);

        List<ApproveUserResult> approveUserResultList = null;
        if (Objects.nonNull(examDetailCourseList) && examDetailCourseList.size() > 0) {
            Set<Long> examDetailIdSet = examDetailCourseList.stream().map(s -> s.getExamDetailId()).collect(Collectors.toSet());
            QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
            examDetailQueryWrapper.lambda().in(ExamDetail::getId, examDetailIdSet);
            List<ExamDetail> examDetailList = examDetailService.list(examDetailQueryWrapper);
            if (Objects.nonNull(examDetailList) && examDetailList.size() > 0) {
                Set<Long> printHouseIdSet = examDetailList.stream().map(s -> s.getPrintHouseId()).collect(Collectors.toSet());
                approveUserResultList = sysUserService.findByOrgIds(printHouseIdSet);
            }
        }
        return approveUserResultList;
    }

    /**
     * 获取流程提交记录
     *
     * @param examTaskId
     * @return
     */
    @Override
    public List<FlowTaskSubmitPeopleAllResult> taskApproverSubmitList(String examTaskId) {
        List<FlowTaskSubmitPeopleAllResult> flowTaskSubmitPeopleAllResultList = null;
        List<TFFlowLogResult> tfFlowLogResultList = tfFlowLogService.findByObjectId(SystemConstant.convertIdToLong(examTaskId));
        if (Objects.nonNull(tfFlowLogResultList) && tfFlowLogResultList.size() > 0) {
            flowTaskSubmitPeopleAllResultList = new LinkedList<>();
            for (TFFlowLogResult t : tfFlowLogResultList) {
                ApproveUserResult submitUser = new ApproveUserResult(t.getOrgName(), t.getOrgType(), t.getLoginName(), t.getRealName(), t.getSchoolId(), t.getCode(), t.getMobileNumber(), t.getCreateTime(), t.getPaperAttachmentId());
                FlowTaskSubmitPeopleAllResult flowTaskSubmitPeopleAllResult = new FlowTaskSubmitPeopleAllResult(t.getFlowId(), submitUser);

                if (Objects.nonNull(t.getPendApproveId())) {
                    List<ApproveUserResult> approveUserList = new LinkedList<>();
                    String[] pendApproveIds = t.getPendApproveId().split(",");
                    for (int i = 0; i < pendApproveIds.length; i++) {
                        approveUserList.addAll(sysUserService.findById(SystemConstant.convertIdToLong(pendApproveIds[i].trim())));
                    }
                    flowTaskSubmitPeopleAllResult.setApproveUserList(approveUserList);
                }
                flowTaskSubmitPeopleAllResultList.add(flowTaskSubmitPeopleAllResult);
            }
        }
        return flowTaskSubmitPeopleAllResultList;
    }

    /**
     * 流程提交校验
     *
     * @param task
     * @param userId
     */
    private void flowSubmitVaild(Task task, Long userId) {
        if (Objects.isNull(task.getAssignee())) {
            Task taskUser = taskService.createTaskQuery().taskId(task.getId()).taskAssignee(String.valueOf(userId)).singleResult();
            Optional.ofNullable(taskUser).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("不能提交他人的任务或者该任务已被审核，请刷新再试！"));
        } else {
            if (!Objects.equals(task.getAssignee(), String.valueOf(userId))) {
                throw ExceptionResultEnum.ERROR.exception("不能提交他人的任务或者该任务已被审核，请刷新再试！");
            }
        }
    }

    /**
     * 流程审批校验
     *
     * @param task
     * @param userId
     * @param map
     */
    private void flowApproveVaild(Task task, Long userId, Map<String, Object> map) {
        if (Objects.isNull(task.getAssignee())) {
            Task taskUser = taskService.createTaskQuery().taskId(task.getId()).taskCandidateUser(String.valueOf(userId)).singleResult();
            Optional.ofNullable(taskUser).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("不能提交他人的任务或者该任务已被审核，请刷新再试！"));
        } else {
            if (!Objects.equals(task.getAssignee(), String.valueOf(userId))) {
                throw ExceptionResultEnum.ERROR.exception("不能提交他人的任务或者该任务已被审核，请刷新再试！");
            }
        }
        Optional.ofNullable(map.get(SystemConstant.APPROVE_OPERATION)).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程审核操作不能为空"));
    }

    /**
     * 获取所有审批人
     *
     * @param courseCode
     * @param flowTaskApprovePeopleResultList
     * @param processDefinitionEntity
     * @param processInstanceId
     * @param taskTaskAll
     */
    private List<FlowTaskApprovePeopleResult> getApproveUserAll(String courseCode
            , List<FlowTaskApprovePeopleResult> flowTaskApprovePeopleResultList
            , String processDefinitionEntity
            , String processInstanceId
            , List<FlowTaskResult> taskTaskAll) {
        flowTaskApprovePeopleResultList = Objects.isNull(flowTaskApprovePeopleResultList) ? new LinkedList() : flowTaskApprovePeopleResultList;
//        //广东医科大学流程
//        if (Objects.nonNull(processDefinitionEntity) && (processDefinitionEntity.contains(SystemConstant.GDYKDX_FLOW_KEY)
//                || processDefinitionEntity.contains(SystemConstant.GDYKDX_SUB_FLOW_KEY))) {
//            //获取教研室主任审批人
//            List<ApproveUserResult> sysUserDirectorList = sysUserService.findByDirectorApprove(courseCode, null);
//            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), false, sysUserDirectorList));
//            Set<Long> orgIds = sysUserDirectorList.stream().map(s -> s.getOrgId()).collect(Collectors.toSet());
//            //获取院长审批人
//            List<ApproveUserResult> sysUserPresidentList = sysUserService.findByPresidentApprove(new ArrayList<>(orgIds), null);
//            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.SECOND_APPROVE.getSetup(), false, sysUserPresidentList));
//            if (Objects.nonNull(taskTaskAll) && taskTaskAll.size() > 3) {
//                //获取命题审批人
//                List<ApproveUserResult> sysUserTeacherList = sysUserService.findByAssignApprove(courseCode, null);
//                flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.THREE_APPROVE.getSetup(), false, sysUserTeacherList));
//                if (Objects.nonNull(processInstanceId)) {
//                    //获取印刷员审批人
//                    List<ApproveUserResult> sysUserPrintList = this.findByPrintApprove(SystemConstant.convertIdToLong(processInstanceId));
//                    flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.FOUR_APPROVE.getSetup(), false, sysUserPrintList));
//                }
//            }
//        }//江西中医药大学
//        else if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.contains(SystemConstant.JXZYY_FLOW_KEY)) {
//            //获取教研室主任审批人
//            List<ApproveUserResult> sysUserDirectorList = sysUserService.findByJxzyyDirectorApprove(null, FlowApproveNameEnum.DIRECTOR);
//            flowTaskApprovePeopleResultList.add(new FlowTaskApprovePeopleResult(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup(), false, sysUserDirectorList));
//        } else {
//            throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//        }
        return flowTaskApprovePeopleResultList;
    }

//    /**
//     * 命题提交
//     *
//     * @param task
//     * @param sysUser
//     * @param tfFlowApprove
//     * @param tfFlowLog
//     * @param map
//     */
//    private void assignSubmit(Task task,
//                              SysUser sysUser,
//                              TFFlowApprove tfFlowApprove,
//                              TFFlowLog tfFlowLog,
//                              Map<String, Object> map) {
//        flowSubmitVaild(task, sysUser.getId());
//        tfFlowLog.setApproveSetup(FlowApproveSetupEnum.SUBMIT.getSetup());
//        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.SUBMIT);
//        tfFlowLog.setPaperAttachmentId((String) map.get(SystemConstant.PAPAER_ATTACHMENT_IDS));
//        tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
//        tfFlowApprove.setSetup(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup());
//        map.computeIfAbsent(SystemConstant.APPROVE_DIRECTOR_USER_IDS, v -> map.get(SystemConstant.APPROVE_USER_IDS));
//    }
//
//    /**
//     * 主任审核
//     *
//     * @param task
//     * @param sysUser
//     * @param tfFlowApprove
//     * @param tfFlowApproveLog
//     * @param tfFlowLog
//     * @param remark
//     * @param processDefinitionEntity
//     * @param map
//     */
//    private void directorApprove(Task task,
//                                 SysUser sysUser,
//                                 TFFlowApprove tfFlowApprove,
//                                 TFFlowApproveLog tfFlowApproveLog,
//                                 TFFlowLog tfFlowLog,
//                                 String remark,
//                                 ProcessDefinitionEntity processDefinitionEntity,
//                                 Map<String, Object> map) {
//        FlowApprovePassEnum flowApprovePassEnum = (FlowApprovePassEnum) map.get(SystemConstant.APPROVE_CANCEL);
//        Long approveCancelUserId = (Long) map.get(SystemConstant.APPROVE_CANCEL_USER);
//        if (Objects.isNull(approveCancelUserId) || approveCancelUserId.longValue() != SystemConstant.APPROVE_CANCEL_USER_ID.longValue()) {
//            flowApproveVaild(task, sysUser.getId(), map);
//        }
//        FlowApprovePassEnum approvePass = (FlowApprovePassEnum) map.get(SystemConstant.APPROVE_OPERATION);
//        FlowApproveVariableEnum approve = approvePass == FlowApprovePassEnum.PASS ? FlowApproveVariableEnum.PRIMARY_APPROVE : FlowApproveVariableEnum.PRIMARY_REJECT;
//
//        tfFlowLog.setApproveSetup(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup());
//        tfFlowApproveLog.setPrimaryApproveId(sysUser.getId());
//        tfFlowApproveLog.setPrimaryApproveRemark(remark);
//        if (approve == FlowApproveVariableEnum.PRIMARY_REJECT) {//驳回命题
//            if (Objects.nonNull(flowApprovePassEnum) && flowApprovePassEnum == FlowApprovePassEnum.CANCEL) {
//                tfFlowLog.setApproveOperation(FlowApproveOperationEnum.CANCEL);
//                tfFlowApproveLog.setPrimaryApproveOperation(FlowApproveOperationEnum.CANCEL);
//                tfFlowApprove.setStatus(FlowStatusEnum.CANCEL);
//                tfFlowLog.setEnable(false);
//
////                List<TFFlowLog> tfFlowLogList = tfFlowLogService.findByFlowId(Long.parseLong(task.getProcessInstanceId()), sysUser.getSchoolId());
//                UpdateWrapper<TFFlowLog> tfFlowLogUpdateWrapper = new UpdateWrapper<>();
//                tfFlowLogUpdateWrapper.lambda().eq(TFFlowLog::getApproveOperation, FlowApproveOperationEnum.SUBMIT)
//                        .eq(TFFlowLog::getFlowId, Long.parseLong(task.getProcessInstanceId()))
////                        .set(TFFlowLog::getPaperAttachmentId, null);
//                        .set(TFFlowLog::getEnable, false);
//                tfFlowLogService.update(tfFlowLogUpdateWrapper);
////                tfFlowLogService.removeByIds(tfFlowLogList.stream().map(s -> s.getId()).collect(Collectors.toList()));
//            } else {
//                tfFlowLog.setApproveOperation(FlowApproveOperationEnum.REJECT);
//                tfFlowApproveLog.setPrimaryApproveOperation(FlowApproveOperationEnum.REJECT);
//                tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
//            }
//            tfFlowApprove.setSetup(FlowApproveSetupEnum.SUBMIT.getSetup());
//            tfFlowLog.setPendApproveId(String.valueOf(tfFlowApproveLog.getStartId()));
//        } else if (approve == FlowApproveVariableEnum.PRIMARY_APPROVE) {//提交
//            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
//            tfFlowApproveLog.setPrimaryApproveOperation(FlowApproveOperationEnum.APPROVE);
//            //广东医科大学流程
//            if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)) {
//                tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
//                tfFlowApprove.setSetup(FlowApproveSetupEnum.SECOND_APPROVE.getSetup());
//            }
//            //江西中医药大学
//            else if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.JXZYY_FLOW_KEY)) {
//                tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
//                tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
//            } else {
//                throw ExceptionResultEnum.ERROR.exception("未配置流程学校code");
//            }
//        } else {
//            throw ExceptionResultEnum.ERROR.exception("流程变量值错误");
//        }
//        map.computeIfAbsent(SystemConstant.APPROVE, v -> approve.getId());
//        map.computeIfAbsent(SystemConstant.APPROVE_PRESIDENT_USER_IDS, v -> map.get(SystemConstant.APPROVE_USER_IDS));
//    }

//    /**
//     * 命题终止
//     *
//     * @param tfFlowApproveLog
//     * @param sysUser
//     */
//    private void assignFlowEnd(TFFlowApproveLog tfFlowApproveLog, SysUser sysUser) {
//        tfFlowApproveLog.setStartId(sysUser.getId());
//        tfFlowApproveLog.setRemark(FlowApproveSetupEnum.SUBMIT.getTitle() + FlowApproveOperationEnum.END.getTitle());
//    }

//    /**
//     * 主任终止
//     *
//     * @param tfFlowApproveLog
//     * @param sysUser
//     */
//    private void directorFlowEnd(TFFlowApproveLog tfFlowApproveLog, SysUser sysUser) {
//        tfFlowApproveLog.setPrimaryApproveId(sysUser.getId());
//        tfFlowApproveLog.setPrimaryApproveOperation(FlowApproveOperationEnum.END);
//        tfFlowApproveLog.setPrimaryApproveRemark(FlowApproveOperationEnum.END.getTitle());
//    }

//    /**
//     * 院长终止
//     *
//     * @param tfFlowApproveLog
//     * @param sysUser
//     */
//    private void presidentFlowEnd(TFFlowApproveLog tfFlowApproveLog, SysUser sysUser) {
//        tfFlowApproveLog.setSecondApproveId(sysUser.getId());
//        tfFlowApproveLog.setSecondApproveOperation(FlowApproveOperationEnum.END);
//        tfFlowApproveLog.setSecondApproveRemark(FlowApproveOperationEnum.END.getTitle());
//    }

//    /**
//     * 命题终止
//     *
//     * @param tfFlowApproveLog
//     * @param sysUser
//     */
//    private void teacherFlowEnd(TFFlowApproveLog tfFlowApproveLog, SysUser sysUser) {
//        tfFlowApproveLog.setThreeApproveId(sysUser.getId());
//        tfFlowApproveLog.setThreeApproveRemark(FlowApproveSetupEnum.THREE_APPROVE.getTitle() + FlowApproveOperationEnum.END.getTitle());
//        tfFlowApproveLog.setThreeApproveOperation(FlowApproveOperationEnum.APPROVE);
//    }

//    /**
//     * 印刷员终止
//     *
//     * @param tfFlowApproveLog
//     * @param sysUser
//     */
//    private void printFlowEnd(TFFlowApproveLog tfFlowApproveLog, SysUser sysUser) {
//        tfFlowApproveLog.setFourApproveId(sysUser.getId());
//        tfFlowApproveLog.setFourApproveOperation(FlowApproveOperationEnum.END);
//        tfFlowApproveLog.setFourApproveRemark(FlowApproveOperationEnum.END.getTitle());
//    }

//    /**
//     * 院长审核
//     *
//     * @param task
//     * @param sysUser
//     * @param tfFlowApprove
//     * @param tfFlowApproveLog
//     * @param tfFlowLog
//     * @param remark
//     * @param map
//     * @param objectMap
//     * @param processInstanceId
//     */
//    private void presidentApprove(Task task,
//                                  SysUser sysUser,
//                                  TFFlowApprove tfFlowApprove,
//                                  TFFlowApproveLog tfFlowApproveLog,
//                                  TFFlowLog tfFlowLog,
//                                  String remark,
//                                  Map<String, Object> map,
//                                  Map<String, Object> objectMap,
//                                  String processInstanceId) {
//        flowApproveVaild(task, sysUser.getId(), map);
//        FlowApprovePassEnum approvePass = (FlowApprovePassEnum) map.get(SystemConstant.APPROVE_OPERATION);
//        FlowApproveVariableEnum approve = null;
//        tfFlowApproveLog.setSecondApproveId(sysUser.getId());
//        tfFlowApproveLog.setSecondApproveRemark(remark);
//        if (approvePass == FlowApprovePassEnum.PASS) {
//            approve = FlowApproveVariableEnum.SECOND_APPROVE;
//        } else {
//            Optional.ofNullable(map.get(SystemConstant.APPROVE_SETUP)).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程驳回节点不能为空"));
//            Integer setup = Integer.parseInt(map.get(SystemConstant.APPROVE_SETUP).toString());
//            if (setup.intValue() < FlowApproveSetupEnum.SUBMIT.getSetup() || setup.intValue() >= FlowApproveSetupEnum.SECOND_APPROVE.getSetup()) {
//                throw ExceptionResultEnum.ERROR.exception("流程驳回节点有误");
//            }
//            approve = setup.intValue() == 1 ? FlowApproveVariableEnum.SECOND_REJECT_ONE : FlowApproveVariableEnum.SECOND_REJECT_TWO;
//        }
//        tfFlowLog.setApproveSetup(FlowApproveSetupEnum.SECOND_APPROVE.getSetup());
//        if (approve == FlowApproveVariableEnum.SECOND_REJECT_ONE) {//驳回命题
//            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.REJECT);
//            tfFlowApproveLog.setSecondApproveOperation(FlowApproveOperationEnum.REJECT);
//            tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
//            tfFlowApprove.setSetup(FlowApproveSetupEnum.SUBMIT.getSetup());
//            tfFlowLog.setPendApproveId(String.valueOf(tfFlowApproveLog.getStartId()));
//        } else if (approve == FlowApproveVariableEnum.SECOND_REJECT_TWO) {//驳回主任
//            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.REJECT);
//            tfFlowApproveLog.setSecondApproveOperation(FlowApproveOperationEnum.REJECT);
//            tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
//            tfFlowApprove.setSetup(FlowApproveSetupEnum.PRIMARY_APPROVE.getSetup());
//        } else if (approve == FlowApproveVariableEnum.SECOND_APPROVE) {//提交
//            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
//            tfFlowApproveLog.setSecondApproveOperation(FlowApproveOperationEnum.APPROVE);
//            List<FlowTaskResult> taskTaskAll = getTaskAll(processInstanceId);
//            if (Objects.nonNull(taskTaskAll) && taskTaskAll.size() > 3) {
//                tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
//                tfFlowApprove.setSetup(FlowApproveSetupEnum.THREE_APPROVE.getSetup());
//                objectMap.computeIfAbsent(SystemConstant.APPROVE_PRESIDENT_SYSTEM, v -> true);
//                tfFlowLog.setPendApproveId(String.valueOf(tfFlowApproveLog.getStartId()));
//            } else {
//                tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
//                tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
//            }
//        } else {
//            throw ExceptionResultEnum.ERROR.exception("流程变量值错误");
//        }
//        FlowApproveVariableEnum finalApprove = approve;
//        map.computeIfAbsent(SystemConstant.APPROVE, v -> finalApprove.getId());
//        TFFlowApproveLog finalTfFlowApproveLog = tfFlowApproveLog;
//        map.computeIfAbsent(SystemConstant.APPROVE_ID, v -> finalTfFlowApproveLog.getStartId());
//    }

//    /**
//     * 命题老师审核
//     *
//     * @param task
//     * @param sysUser
//     * @param tfFlowApprove
//     * @param tfFlowApproveLog
//     * @param tfFlowLog
//     * @param remark
//     * @param map
//     */
//    private void teacherApprove(Task task,
//                                SysUser sysUser,
//                                TFFlowApprove tfFlowApprove,
//                                TFFlowApproveLog tfFlowApproveLog,
//                                TFFlowLog tfFlowLog,
//                                String remark,
//                                Map<String, Object> map) {
//        boolean presidentApprove = Objects.nonNull(map.get(SystemConstant.APPROVE_PRESIDENT_SYSTEM)) ? (boolean) map.get(SystemConstant.APPROVE_PRESIDENT_SYSTEM) : false;
//        Long userId = null;
//        if (presidentApprove) {
//            userId = tfFlowApproveLog.getStartId();
//        } else {
//            userId = sysUser.getId();
//        }
//        flowSubmitVaild(task, userId);
//        tfFlowLog.setApproveSetup(FlowApproveSetupEnum.THREE_APPROVE.getSetup());
//        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
//        tfFlowLog.setApproveId(userId);
//        tfFlowLog.setPaperAttachmentId((String) map.get(SystemConstant.PAPAER_ATTACHMENT_IDS));
//        tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
//        tfFlowApprove.setSetup(FlowApproveSetupEnum.FOUR_APPROVE.getSetup());
//        tfFlowApproveLog.setThreeApproveId(userId);
//        tfFlowApproveLog.setThreeApproveRemark(remark);
//        tfFlowApproveLog.setThreeApproveOperation(FlowApproveOperationEnum.APPROVE);
//        map.computeIfAbsent(SystemConstant.APPROVE_PRINT_USER_IDS, v -> map.get(SystemConstant.APPROVE_USER_IDS));
//    }

//    /**
//     * 印刷员审核
//     *
//     * @param task
//     * @param sysUser
//     * @param tfFlowApprove
//     * @param tfFlowApproveLog
//     * @param tfFlowLog
//     * @param remark
//     * @param map
//     */
//    private void printApprove(Task task,
//                              SysUser sysUser,
//                              TFFlowApprove tfFlowApprove,
//                              TFFlowApproveLog tfFlowApproveLog,
//                              TFFlowLog tfFlowLog,
//                              String remark,
//                              Map<String, Object> map) {
//        flowApproveVaild(task, sysUser.getId(), map);
//        FlowApprovePassEnum approvePass = (FlowApprovePassEnum) map.get(SystemConstant.APPROVE_OPERATION);
//        FlowApproveVariableEnum approve = approvePass == FlowApprovePassEnum.PASS ? FlowApproveVariableEnum.FOUR_APPROVE : FlowApproveVariableEnum.FOUR_REJECT;
//        tfFlowLog.setApproveSetup(FlowApproveSetupEnum.FOUR_APPROVE.getSetup());
//        tfFlowApproveLog.setFourApproveId(sysUser.getId());
//        tfFlowApproveLog.setFourApproveRemark(remark);
//        if (approve == FlowApproveVariableEnum.FOUR_REJECT) {//驳回命题
//            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.REJECT);
//            tfFlowApproveLog.setFourApproveOperation(FlowApproveOperationEnum.REJECT);
//            tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
//            tfFlowApprove.setSetup(FlowApproveSetupEnum.THREE_APPROVE.getSetup());
//            tfFlowLog.setPendApproveId(String.valueOf(tfFlowApproveLog.getStartId()));
//        } else if (approve == FlowApproveVariableEnum.FOUR_APPROVE) {//提交
//            tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
//            tfFlowApproveLog.setFourApproveOperation(FlowApproveOperationEnum.APPROVE);
//            tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
//            tfFlowApprove.setSetup(FlowApproveSetupEnum.FINISH.getSetup());
//        } else {
//            throw ExceptionResultEnum.ERROR.exception("流程变量值错误");
//        }
//        map.computeIfAbsent(SystemConstant.APPROVE, v -> approve.getId());
//    }

    /**
     * 动态创建流程图
     *
     * @param customFlowSaveDto
     * @param id
     * @param version
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public Map<String, Object> dynamicBuildBpmn(CustomFlowSaveDto customFlowSaveDto, String id, Integer version) throws IOException {
        BpmnModel model = new BpmnModel();
        Process process = new Process();
        model.addProcess(process);
        process.setId(id);

        List<CustomFlowDto> customFlowLists = customFlowSaveDto.getCustomFlowLists();
        Map<CustomFlowTypeEnum, CustomFlowDto> customFlowTypeEnumCustomFlowDtoMap = new HashMap<>();
        FlowTaskLink flowTaskLink = new FlowTaskLink();//流程双向链表
        Map<String, Object> flowProcessVarMap = new HashMap<>();//流程变量
        Map<String, CustomFlowVarDto> agginessMap = new HashMap<>();//流程审核人数据
        Map<String, CustomFlowGatewayDto> approveDataMap = new HashMap<>();//流程审批数据
        Map<String, FlowTaskResult> setupMap = new HashMap<>();//流程审批步骤map
        LinkedMultiValueMap<String, CustomFlowSequenceDto> approvePassMap = new LinkedMultiValueMap<>();//流程审批通过数据
        LinkedMultiValueMap<String, CustomFlowSequenceDto> approveRejectMap = new LinkedMultiValueMap<>();//流程审批驳回数据
        flowProcessVarMap.computeIfAbsent(SystemConstant.AGGINESS_MAP, v -> agginessMap);
        flowProcessVarMap.computeIfAbsent(SystemConstant.APPROVE_PASS_MAP, v -> approvePassMap);
        flowProcessVarMap.computeIfAbsent(SystemConstant.APPROVE_REJECT_MAP, v -> approveRejectMap);
        flowProcessVarMap.computeIfAbsent(SystemConstant.SETUP_MAP, v -> setupMap);
        AtomicInteger gatewayId = new AtomicInteger(1);//流程网关id
        AtomicInteger sequenceId = new AtomicInteger(1);//流程连接id
        AtomicInteger setup = new AtomicInteger(2);//流程步骤id
        CustomFlowDto customFlowDefaultDto = null;
        for (int i = 0; i < customFlowLists.size(); i++) {
            CustomFlowDto customFlowDto = customFlowLists.get(i);
            FlowTaskNode node = new FlowTaskNode(customFlowDto);
            flowTaskLink.add(node);
            switch (customFlowDto.getType()) {
                case START://开始节点
                    if (customFlowTypeEnumCustomFlowDtoMap.containsKey(CustomFlowTypeEnum.START)) {
                        throw ExceptionResultEnum.ERROR.exception("流程只能有一个开始节点！");
                    } else {
                        customFlowTypeEnumCustomFlowDtoMap.computeIfAbsent(CustomFlowTypeEnum.START, v -> customFlowDto);
                    }
                    customFlowDto.setFlowTaskId(CustomFlowTypeEnum.START.getId());
                    process.addFlowElement(createStartEvent());
                    //默认在开始节点后新增一个审批节点
                    customFlowDefaultDto = multiWorkFlow.createDefaultUserTask(customFlowDto, flowTaskLink, DefaultInstanceConvertToMultiInstance.DEFAULT_USER_TASK);
                    setupMap.put(customFlowDefaultDto.getFlowTaskId(), new FlowTaskResult(customFlowDefaultDto.getContent(), customFlowDefaultDto.getFlowTaskId(), customFlowDefaultDto.getSetup()));
                    agginessMap.computeIfAbsent(customFlowDefaultDto.getFlowTaskId(), v -> new CustomFlowVarDto(SystemConstant.APPROVE_ID, new ArrayList(Arrays.asList(""))));
                    process.addFlowElement(createUserTask(CustomFlowTypeEnum.USER_TASK.getId() + DefaultInstanceConvertToMultiInstance.DEFAULT_USER_TASK, Objects.nonNull(flowTaskLink.getLast().getTask().getContent()) && !Objects.equals(flowTaskLink.getLast().getTask().getContent(), "") ? flowTaskLink.getLast().getTask().getContent() : DefaultInstanceConvertToMultiInstance.DEFAULT_USER_TASK_NAME, DefaultInstanceConvertToMultiInstance.APPROVE_ID_EXP));
                    break;
                case PROCESS://过程节点
                    customFlowDto.setSetup(setup.getAndIncrement());
                    customFlowDto.setFlowTaskId(CustomFlowTypeEnum.USER_TASK.getId() + i);
                    setupMap.put(customFlowDto.getFlowTaskId(), new FlowTaskResult(Objects.nonNull(customFlowDto.getContent()) && !Objects.equals(customFlowDto.getContent(), "") ? customFlowDto.getContent() : DefaultInstanceConvertToMultiInstance.USER_TASK_APPROVE_NAME, customFlowDto.getFlowTaskId(), customFlowDto.getSetup()));
                    CustomFlowPropertyDto customFlowPropertyDto = customFlowDto.getProperty();
                    List<String> approveUserIds = new ArrayList<>();//审批用户
                    List<String> copyUserIds = new ArrayList<>();//抄送用户
                    if (Objects.nonNull(customFlowDto.getProperty())) {
                        //选人属性
                        switch (customFlowPropertyDto.getApproveUserType()) {
                            case USER://用户
                                if (Objects.nonNull(customFlowPropertyDto.getApproveUsers()) && customFlowPropertyDto.getApproveUsers().size() > 0) {
                                    approveUserIds.addAll(customFlowPropertyDto.getApproveUsers().stream().map(x -> String.valueOf(x.getId())).collect(Collectors.toList()));
                                }
                                break;
                            case ROLE://角色
                                if (Objects.nonNull(customFlowPropertyDto.getApproveRoles()) && customFlowPropertyDto.getApproveRoles().size() > 0) {
                                    List<SysUser> sysUserList = sysUserService.findByRoleIds(customFlowPropertyDto.getApproveRoles().stream().map(x -> x.getId()).collect(Collectors.toList()));
                                    approveUserIds.addAll(sysUserList.stream().map(x -> String.valueOf(x.getId())).collect(Collectors.toList()));
                                }
                                break;
                            default:
                                break;
                        }
                        ApproveUserResult approveUserResult = sysUserService.findByIds(approveUserIds.stream().map(x -> SystemConstant.convertIdToLong(x)).collect(Collectors.toList()));
                        agginessMap.put(node.getTask().getFlowTaskId(), new CustomFlowVarDto(DefaultInstanceConvertToMultiInstance.DEFAULT_ASSIGNEE_LIST + i, approveUserIds, node.getTask().getProperty().getApproveUserType(), node.getTask().getProperty().getMultipleUserApproveType(), node.getTask().getProperty().getRejectType(), node.getTask().getProperty().getRejectResubmitType(), approveUserResult.getPendApproveUsers()));
                        if (Objects.nonNull(customFlowPropertyDto.getRejectType()) && !Objects.equals(customFlowPropertyDto.getRejectType(), "")) {
                            //驳回属性
                            switch (customFlowPropertyDto.getRejectType()) {
                                case PREV://上一节点
                                    String flowTaskId = null;
                                    Integer flowSetup = null;
                                    FlowTaskNode flowTaskNode = node.getBefore();
                                    boolean isProcess = false;
                                    while (!isProcess) {
                                        if (flowTaskNode.getTask().getType() == CustomFlowTypeEnum.PROCESS) {
                                            isProcess = true;
                                            flowTaskId = flowTaskNode.getTask().getFlowTaskId();
                                            flowSetup = flowTaskNode.getTask().getSetup();
                                        }
                                        flowTaskNode = flowTaskNode.getBefore();
                                    }
                                    this.createCustomFlowExclusiveGatewayAfter(node,
                                            flowTaskLink,
                                            approveDataMap,
                                            process,
                                            gatewayId,
                                            flowTaskId,
                                            false,
                                            flowSetup);
                                    break;
                                case START://发起人节点
                                    this.createCustomFlowExclusiveGatewayAfter(node,
                                            flowTaskLink,
                                            approveDataMap,
                                            process,
                                            gatewayId,
                                            customFlowDefaultDto.getFlowTaskId(),
                                            false,
                                            customFlowDefaultDto.getSetup());
                                    break;
                                case PREV_ALL://该节点前全部节点
                                    flowTaskNode = node.getBefore();
                                    while (Objects.nonNull(flowTaskNode)) {
                                        if (flowTaskNode.getTask().getType() == CustomFlowTypeEnum.PROCESS) {
                                            flowTaskId = flowTaskNode.getTask().getFlowTaskId();
                                            flowSetup = flowTaskNode.getTask().getSetup();
                                            this.createCustomFlowExclusiveGatewayAfter(node,
                                                    flowTaskLink,
                                                    approveDataMap,
                                                    process,
                                                    gatewayId,
                                                    flowTaskId,
                                                    false,
                                                    flowSetup);
                                        }
                                        flowTaskNode = flowTaskNode.getBefore();
                                    }
                                    break;
                                default:
                                    break;
                            }
                            //驳回再提交属性
                            switch (customFlowPropertyDto.getRejectResubmitType()) {
                                case PREV_STEP://提交到驳回节点
                                    CustomFlowGatewayDto customFlowGatewayDto = approveDataMap.get(node.getTask().getFlowTaskId());
                                    Map<String, List<CustomFlowSequenceDto>> flowTaskIds = customFlowGatewayDto.getFlowTaskIds();
                                    List<CustomFlowSequenceDto> customFlowSequenceDtoList = flowTaskIds.get(customFlowGatewayDto.getAfterExclusiveGatewayId());
                                    for (int y = 1; y < flowTaskLink.size(); y++) {
                                        FlowTaskNode nodeTemp = flowTaskLink.get(y);
                                        for (CustomFlowSequenceDto c : customFlowSequenceDtoList) {
                                            if (Objects.equals(nodeTemp.getTask().getFlowTaskId(), c.getAfterFlowTaskId())) {
                                                if (nodeTemp.getNext().getTask().getType() == CustomFlowTypeEnum.EXCLUSIVE_GATE_WAY) {
                                                    this.createCustomFlowExclusiveGatewayBefore(node,
                                                            flowTaskLink,
                                                            y + 1,
                                                            approveDataMap,
                                                            process,
                                                            gatewayId,
                                                            node.getTask().getFlowTaskId(),
                                                            true,
                                                            nodeTemp.getNext(),
                                                            node.getTask().getSetup());
                                                } else {
                                                    this.createCustomFlowExclusiveGatewayBefore(node,
                                                            flowTaskLink,
                                                            y + 1,
                                                            approveDataMap,
                                                            process,
                                                            gatewayId,
                                                            node.getTask().getFlowTaskId(),
                                                            true,
                                                            null,
                                                            node.getTask().getSetup());
                                                }
                                            }
                                        }
                                    }
                                    break;
                                default:
                                    break;
                            }
                        }
                        //抄送用户
                        if (Objects.nonNull(customFlowPropertyDto.getCopyForUsers())) {
                            copyUserIds.addAll(customFlowPropertyDto.getCopyForUsers().stream().map(x -> String.valueOf(x.getId())).collect(Collectors.toList()));
                        }
                    }
                    process.addFlowElement(createUserTask(CustomFlowTypeEnum.USER_TASK.getId() + i, Objects.nonNull(customFlowDto.getContent()) && !Objects.equals(customFlowDto.getContent(), "") ? customFlowDto.getContent() : DefaultInstanceConvertToMultiInstance.USER_TASK_APPROVE_NAME, approveUserIds, customFlowPropertyDto.getMultipleUserApproveType()));
                    break;
                case END://结束节点
                    if (customFlowTypeEnumCustomFlowDtoMap.containsKey(CustomFlowTypeEnum.END)) {
                        throw ExceptionResultEnum.ERROR.exception("流程只能有一个结束节点！");
                    } else {
                        customFlowTypeEnumCustomFlowDtoMap.computeIfAbsent(CustomFlowTypeEnum.END, v -> customFlowDto);
                    }
                    customFlowDto.setSetup(0);
                    customFlowDto.setFlowTaskId(CustomFlowTypeEnum.END.getId());
                    setupMap.put(customFlowDto.getFlowTaskId(), new FlowTaskResult(customFlowDto.getContent(), customFlowDto.getFlowTaskId(), customFlowDto.getSetup()));
                    process.addFlowElement(createEndEvent());
                    break;
                default:
                    break;
            }
        }

        for (int i = 1; i < flowTaskLink.size(); i++) {
            FlowTaskNode flowTaskNode = flowTaskLink.get(i);
            String normalSequenceId = DefaultInstanceConvertToMultiInstance.SEQUENCE_NAME + sequenceId.getAndIncrement();
            if (flowTaskNode.getBefore().getTask().getType() == CustomFlowTypeEnum.EXCLUSIVE_GATE_WAY) {
                String conditionExp = DefaultInstanceConvertToMultiInstance.EXP_PREFIX + SystemConstant.APPROVE + DefaultInstanceConvertToMultiInstance.EXP_EQUAL + "'" + flowTaskNode.getTask().getFlowTaskId() + "'" + DefaultInstanceConvertToMultiInstance.EXP_SUFFIX;
                if (flowTaskNode.getBefore().getTask().getType() == CustomFlowTypeEnum.EXCLUSIVE_GATE_WAY) {
                    approvePassMap.add(flowTaskNode.getBefore().getBefore().getTask().getFlowTaskId(), new CustomFlowSequenceDto(flowTaskNode.getTask().getFlowTaskId(), conditionExp, true, flowTaskNode.getTask().getSetup()));
                } else {
                    approvePassMap.add(flowTaskNode.getBefore().getTask().getFlowTaskId(), new CustomFlowSequenceDto(flowTaskNode.getTask().getFlowTaskId(), conditionExp, true, flowTaskNode.getTask().getSetup()));
                }
                log.info("1 start:{},end:{}", flowTaskNode.getBefore().getTask().getFlowTaskId(), flowTaskNode.getTask().getFlowTaskId());
                process.addFlowElement(createSequenceFlow(flowTaskNode.getBefore().getTask().getFlowTaskId(), flowTaskNode.getTask().getFlowTaskId(), normalSequenceId, FlowApprovePassEnum.PASS.getTitle(), conditionExp));
            } else {
                if (flowTaskNode.getBefore().getTask().getType() == CustomFlowTypeEnum.EXCLUSIVE_GATE_WAY) {
                    approvePassMap.add(flowTaskNode.getBefore().getBefore().getTask().getFlowTaskId(), new CustomFlowSequenceDto(flowTaskNode.getTask().getFlowTaskId(), true, flowTaskNode.getTask().getSetup()));
                } else {
                    approvePassMap.add(flowTaskNode.getBefore().getTask().getFlowTaskId(), new CustomFlowSequenceDto(flowTaskNode.getTask().getFlowTaskId(), true, flowTaskNode.getTask().getSetup()));
                }
                log.info("2 start:{},end:{}", flowTaskNode.getBefore().getTask().getFlowTaskId(), flowTaskNode.getTask().getFlowTaskId());
                process.addFlowElement(createSequenceFlow(flowTaskNode.getBefore().getTask().getFlowTaskId(), flowTaskNode.getTask().getFlowTaskId(), normalSequenceId, FlowApprovePassEnum.PASS.getTitle(), null));
            }
            List<CustomFlowSequenceDto> beforeFlowTaskIdsList = flowTaskNode.getTask().getBeforeFlowTaskIds();
            if (Objects.nonNull(beforeFlowTaskIdsList)) {
                for (CustomFlowSequenceDto c : beforeFlowTaskIdsList) {
                    String gatewaySequenceId = DefaultInstanceConvertToMultiInstance.SEQUENCE_NAME + sequenceId.getAndIncrement();
                    if (Objects.nonNull(c.getBeforeFlowTaskId())) {
                        approvePassMap.add(flowTaskNode.getBefore().getTask().getFlowTaskId(), c);
                        log.info("3 start:{},end:{}", flowTaskNode.getTask().getFlowTaskId(), c.getBeforeFlowTaskId());
                        process.addFlowElement(createSequenceFlow(flowTaskNode.getTask().getFlowTaskId(), c.getBeforeFlowTaskId(), gatewaySequenceId, FlowApprovePassEnum.PASS.getTitle(), c.getConditionExp()));
                    }
                }
            }
            List<CustomFlowSequenceDto> afterFlowTaskIdsList = flowTaskNode.getTask().getAfterFlowTaskIds();
            if (Objects.nonNull(afterFlowTaskIdsList)) {
                for (CustomFlowSequenceDto c : afterFlowTaskIdsList) {
                    String gatewaySequenceId = DefaultInstanceConvertToMultiInstance.SEQUENCE_NAME + sequenceId.getAndIncrement();
                    if (Objects.nonNull(c.getAfterFlowTaskId())) {
                        approveRejectMap.add(flowTaskNode.getBefore().getTask().getFlowTaskId(), c);
                        log.info("4 start:{},end:{}", flowTaskNode.getTask().getFlowTaskId(), c.getAfterFlowTaskId());
                        process.addFlowElement(createSequenceFlow(flowTaskNode.getTask().getFlowTaskId(), c.getAfterFlowTaskId(), gatewaySequenceId, FlowApprovePassEnum.REJECT.getTitle(), c.getConditionExp()));
                    }
                }
            }
        }
        //生成图像信息
        new BpmnAutoLayout(model).execute();

        Map<String, Object> map = new HashMap<>();
        map.computeIfAbsent(SystemConstant.ID, v -> id);
        map.computeIfAbsent(SystemConstant.MODEL, v -> model);
        map.computeIfAbsent(SystemConstant.FLOW_PROCESS_VAR_MAP, v -> flowProcessVarMap);
        map.computeIfAbsent(SystemConstant.VERSION, v -> version);
        this.flowDeployment(map);
        return map;
    }

    /**
     * 创建当前节点驳回网关
     *
     * @param node
     * @param flowTaskLink
     * @param approveVarMap
     * @param process
     * @param gatewayId
     * @param flowTaskId
     * @param pass
     */
    protected void createCustomFlowExclusiveGatewayAfter(FlowTaskNode node,
                                                         FlowTaskLink flowTaskLink,
                                                         Map<String, CustomFlowGatewayDto> approveVarMap,
                                                         Process process,
                                                         AtomicInteger gatewayId,
                                                         String flowTaskId,
                                                         boolean pass,
                                                         Integer setup) {
        CustomFlowGatewayDto customFlowGatewayDto = null;
        FlowTaskNode customFlowExclusiveGatewayTaskNode = null;
        List<CustomFlowSequenceDto> customFlowSequenceDtoList = null;
        if (approveVarMap.containsKey(node.getTask().getFlowTaskId())) {
            customFlowGatewayDto = approveVarMap.get(node.getTask().getFlowTaskId());
        } else {
            customFlowGatewayDto = new CustomFlowGatewayDto();
        }
        //是否存在驳回网关
        if (Objects.isNull(customFlowGatewayDto.getAfterExclusiveGatewayId())) {
            customFlowExclusiveGatewayTaskNode = createCustomFlowExclusiveGateway(flowTaskLink, -1, process, gatewayId);
            customFlowGatewayDto.setAfterExclusiveGatewayId(customFlowExclusiveGatewayTaskNode.getTask().getFlowTaskId());
        } else {
            customFlowExclusiveGatewayTaskNode = node.getNext();
        }
        CustomFlowSequenceDto customFlowSequenceRejectDto = new CustomFlowSequenceDto(flowTaskId, DefaultInstanceConvertToMultiInstance.EXP_PREFIX + SystemConstant.APPROVE + DefaultInstanceConvertToMultiInstance.EXP_EQUAL + "'" + flowTaskId + "'" + DefaultInstanceConvertToMultiInstance.EXP_SUFFIX, pass, setup);
        Map<String, List<CustomFlowSequenceDto>> flowTaskIds = customFlowGatewayDto.getFlowTaskIds();
        flowTaskIds = Objects.isNull(flowTaskIds) ? new HashMap<>() : flowTaskIds;
        if (flowTaskIds.containsKey(customFlowGatewayDto.getAfterExclusiveGatewayId())) {
            customFlowSequenceDtoList = flowTaskIds.get(customFlowGatewayDto.getAfterExclusiveGatewayId());
        } else {
            customFlowSequenceDtoList = new ArrayList<>();
        }
        customFlowSequenceDtoList.add(customFlowSequenceRejectDto);
        flowTaskIds.put(customFlowGatewayDto.getAfterExclusiveGatewayId(), customFlowSequenceDtoList);
        customFlowGatewayDto.setFlowTaskIds(flowTaskIds);
        approveVarMap.put(node.getTask().getFlowTaskId(), customFlowGatewayDto);
        customFlowExclusiveGatewayTaskNode.getTask().setAfterFlowTaskIds(flowTaskIds.get(customFlowExclusiveGatewayTaskNode.getTask().getFlowTaskId()));
    }

    /**
     * 创建当前节点审批网关
     *
     * @param node
     * @param flowTaskLink
     * @param index
     * @param approveDataMap
     * @param process
     * @param gatewayId
     * @param flowTaskId
     * @param pass
     * @param exclusiveGatewayTaskNode
     * @param setup
     */
    protected void createCustomFlowExclusiveGatewayBefore(FlowTaskNode node,
                                                          FlowTaskLink flowTaskLink,
                                                          int index,
                                                          Map<String, CustomFlowGatewayDto> approveDataMap,
                                                          Process process,
                                                          AtomicInteger gatewayId,
                                                          String flowTaskId,
                                                          boolean pass,
                                                          FlowTaskNode exclusiveGatewayTaskNode,
                                                          Integer setup) {
        CustomFlowGatewayDto customFlowGatewayDto = null;
        FlowTaskNode customFlowExclusiveGatewayTaskNode = exclusiveGatewayTaskNode;
        List<CustomFlowSequenceDto> customFlowSequenceDtoList = null;
        if (approveDataMap.containsKey(node.getTask().getFlowTaskId())) {
            customFlowGatewayDto = approveDataMap.get(node.getTask().getFlowTaskId());
        } else {
            customFlowGatewayDto = new CustomFlowGatewayDto();
        }
        //是否存在审批网关
        if (Objects.isNull(customFlowExclusiveGatewayTaskNode) && (Objects.isNull(customFlowGatewayDto.getBeforeExclusiveGatewayId()) || customFlowGatewayDto.getBeforeExclusiveGatewayId().size() == 0)) {
            customFlowExclusiveGatewayTaskNode = createCustomFlowExclusiveGateway(flowTaskLink, index, process, gatewayId);
        }
        List<String> beforeExclusiveGatewayIdList = customFlowGatewayDto.getBeforeExclusiveGatewayId();
        beforeExclusiveGatewayIdList.add(customFlowExclusiveGatewayTaskNode.getTask().getFlowTaskId());
        customFlowGatewayDto.setBeforeExclusiveGatewayId(beforeExclusiveGatewayIdList);

        CustomFlowSequenceDto customFlowSequenceRejectDto = new CustomFlowSequenceDto(flowTaskId, DefaultInstanceConvertToMultiInstance.EXP_PREFIX + SystemConstant.APPROVE + DefaultInstanceConvertToMultiInstance.EXP_EQUAL + "'" + flowTaskId + "'" + DefaultInstanceConvertToMultiInstance.EXP_SUFFIX, pass, setup);
        Map<String, List<CustomFlowSequenceDto>> flowTaskIds = customFlowGatewayDto.getFlowTaskIds();
        flowTaskIds = Objects.isNull(flowTaskIds) ? new HashMap<>() : flowTaskIds;
        if (flowTaskIds.containsKey(customFlowGatewayDto.getBeforeExclusiveGatewayId())) {
            customFlowSequenceDtoList = flowTaskIds.get(customFlowGatewayDto.getBeforeExclusiveGatewayId());
        } else {
            customFlowSequenceDtoList = new ArrayList<>();
        }
        customFlowSequenceDtoList.add(customFlowSequenceRejectDto);
        flowTaskIds.put(customFlowExclusiveGatewayTaskNode.getTask().getFlowTaskId(), customFlowSequenceDtoList);
        customFlowGatewayDto.setFlowTaskIds(flowTaskIds);
        approveDataMap.put(node.getTask().getFlowTaskId(), customFlowGatewayDto);
        List<CustomFlowSequenceDto> customFlowSequenceGateWayDtoList = customFlowExclusiveGatewayTaskNode.getTask().getBeforeFlowTaskIds();
        customFlowSequenceGateWayDtoList.addAll(flowTaskIds.get(customFlowExclusiveGatewayTaskNode.getTask().getFlowTaskId()));
        customFlowExclusiveGatewayTaskNode.getTask().setBeforeFlowTaskIds(customFlowSequenceGateWayDtoList);
    }

    /**
     * 创建自定义流程排他网关dto
     *
     * @param flowTaskLink
     * @param index
     * @param process
     * @param gatewayId
     * @return
     */
    protected FlowTaskNode createCustomFlowExclusiveGateway(FlowTaskLink flowTaskLink,
                                                            int index,
                                                            Process process,
                                                            AtomicInteger gatewayId) {
        int exclusiveGatewayId = gatewayId.getAndIncrement();
        CustomFlowDto customFlowExclusiveGatewayDto = new CustomFlowDto(DefaultInstanceConvertToMultiInstance.GATEWAY_NAME + exclusiveGatewayId, CustomFlowTypeEnum.EXCLUSIVE_GATE_WAY, CustomFlowTypeEnum.EXCLUSIVE_GATE_WAY.getTitle(), CustomFlowTypeEnum.EXCLUSIVE_GATE_WAY.getId() + exclusiveGatewayId);
        FlowTaskNode exclusiveGatewayNode = new FlowTaskNode(customFlowExclusiveGatewayDto);
        if (index <= -1) {
            flowTaskLink.add(exclusiveGatewayNode);
        } else {
            flowTaskLink.add(index, exclusiveGatewayNode);
        }
        process.addFlowElement(createExclusiveGateway(customFlowExclusiveGatewayDto.getFlowTaskId(), CustomFlowTypeEnum.EXCLUSIVE_GATE_WAY.getTitle()));
        return exclusiveGatewayNode;
    }

    /**
     * 启动自定义流程
     *
     * @param map
     */
    @Override
    @Transactional
    public Map<String, Object> customFlowStart(Map<String, Object> map) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long approveId = (Long) map.get(SystemConstant.APPROVE_ID);
        TFCustomFlow tfCustomFlow = tfCustomFlowService.findMaxVersion(sysUser.getSchoolId(), null, TFCustomTypeEnum.ELECTRON_FLOW);
        Optional.ofNullable(tfCustomFlow).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到自定义流程数据"));

        TFCustomFlowEntity tfCustomFlowEntity = new TFCustomFlowEntity(tfCustomFlow.getId(), sysUser.getId());
        Map<String, Object> flowProcessVarMap = this.getFlowProcessVarMap(tfCustomFlow);
        Map<String, CustomFlowVarDto> agginessMap = (Map<String, CustomFlowVarDto>) flowProcessVarMap.get(SystemConstant.AGGINESS_MAP);
        List<String> approveUserIds = new ArrayList(Arrays.asList(String.valueOf(approveId)));
        ApproveUserResult approveUserResult = sysUserService.findByIds(approveUserIds.stream().map(x -> SystemConstant.convertIdToLong(x)).collect(Collectors.toList()));
        agginessMap.put(CustomFlowTypeEnum.USER_TASK.getId() + DefaultInstanceConvertToMultiInstance.DEFAULT_USER_TASK, new CustomFlowVarDto(SystemConstant.APPROVE_ID, approveUserIds, approveUserResult.getPendApproveUsers()));
        flowProcessVarMap.put(SystemConstant.AGGINESS_MAP, agginessMap);

        map.computeIfAbsent(SystemConstant.ID, v -> tfCustomFlow.getFlowKey());
        map.computeIfAbsent(SystemConstant.FLOW_PROCESS_VAR_MAP, v -> flowProcessVarMap);
        map.computeIfAbsent(SystemConstant.FLOW_ENTITY_ID, v -> String.valueOf(tfCustomFlowEntity.getId()));
        map.computeIfAbsent(SystemConstant.CUSTOM_FLOW_ID, v -> tfCustomFlow.getId());
        map.computeIfAbsent(SystemConstant.VERSION, v -> tfCustomFlow.getVersion());
        this.flowStart(map);

        tfCustomFlowEntity.setFlowId(SystemConstant.convertIdToLong((String) map.get(SystemConstant.FLOW_ID)));

        map.remove(SystemConstant.APPROVE_ID);
        map.remove(SystemConstant.PROCESS_DEFINITION_ID);
        map.remove(SystemConstant.FLOW_STATUS);
        map.computeIfAbsent(SystemConstant.ACT_FLOW_ID, v -> tfCustomFlow.getActFlowId());
        tfCustomFlowEntity.setFlowProcessVar(JacksonUtil.parseJson(map));
        tfCustomFlow.setUpdateId(sysUser.getId());
        tfCustomFlowEntityService.save(tfCustomFlowEntity);
        tfCustomFlowService.updateById(tfCustomFlow);
        return map;
    }

    /**
     * 获取当前流程节点信息
     *
     * @param taskId
     * @return
     */
    @Override
    public TaskInfoResult getTaskInfo(Long taskId) {
        Task task = taskService.createTaskQuery().taskId(String.valueOf(taskId)).singleResult();
        Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程任务为空或该流程已被他人审核，请刷新再试！"));
        Long flowId = SystemConstant.convertIdToLong(task.getProcessInstanceId());
        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.findByFlowId(flowId);
        Map<String, Object> flowProcessVarMap = this.getFlowProcessVarMap(tfCustomFlowEntity);
        Map<String, List<CustomFlowSequenceDto>> approveRejectMap = (Map<String, List<CustomFlowSequenceDto>>) flowProcessVarMap.get(SystemConstant.APPROVE_REJECT_MAP);
        Optional.ofNullable(approveRejectMap).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有驳回数据！"));
        Map<String, FlowTaskResult> setupMap = (Map<String, FlowTaskResult>) flowProcessVarMap.get(SystemConstant.SETUP_MAP);
        Optional.ofNullable(setupMap).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程没有步骤数据！"));

        Gson gson = new Gson();
        FlowTaskResult currFlowTaskResult = gson.fromJson(gson.toJson(setupMap.get(task.getTaskDefinitionKey())), FlowTaskResult.class);

        TaskInfoResult taskInfoResult = new TaskInfoResult(flowId, task.getName(), task.getTaskDefinitionKey(), currFlowTaskResult.getSetup());
        List<FlowTaskResult> flowTaskResultList = null;
        if (Objects.nonNull(approveRejectMap.get(task.getTaskDefinitionKey()))) {
            flowTaskResultList = new ArrayList<>();
            List<CustomFlowSequenceDto> customFlowSequenceDtoList = gson.fromJson(gson.toJson(approveRejectMap.get(task.getTaskDefinitionKey())), new TypeToken<List<CustomFlowSequenceDto>>() {
            }.getType());
            for (CustomFlowSequenceDto c : customFlowSequenceDtoList) {
                FlowTaskResult flowTaskResult = gson.fromJson(gson.toJson(setupMap.get(c.getAfterFlowTaskId())), FlowTaskResult.class);
                flowTaskResult.setFlowId(flowId);
                flowTaskResultList.add(flowTaskResult);
            }
        }
        taskInfoResult.setRejectSetupList(flowTaskResultList);
        return taskInfoResult;
    }

    /**
     * 获取当前流程信息
     *
     * @param flowId
     * @return
     */
    @Override
    public FlowViewResult getFlowView(Long flowId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.findByFlowId(flowId);
        Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("自定义流程实体数据为空"));

        //获取流程id
        List<TFFlowJoin> tfFlowJoinList = tfFlowJoinService.findByFlowEntityId(tfCustomFlowEntity.getId());
        Optional.ofNullable(tfFlowJoinList).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程数据为空"));

        //获取当前流程节点
        TFFlowApprove tfFlowApprove = tfFlowApproveService.findByFlowId(flowId, sysUser.getSchoolId());
        Optional.ofNullable(tfFlowApprove).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("流程审核数据为空"));

        Map<String, Object> flowProcessVarMap = this.getFlowProcessVarMap(tfCustomFlowEntity);
        Map<String, FlowTaskResult> setupMap = (Map<String, FlowTaskResult>) flowProcessVarMap.get(SystemConstant.SETUP_MAP);

        Gson gson = new Gson();
        FlowTaskResult currFlowTaskResult = null;
        for (Map.Entry<String, FlowTaskResult> entry : setupMap.entrySet()) {
            FlowTaskResult flowTaskResult = gson.fromJson(gson.toJson(entry.getValue()), FlowTaskResult.class);
            if (Objects.nonNull(tfFlowApprove.getSetup()) && flowTaskResult.getSetup().intValue() == tfFlowApprove.getSetup().intValue()) {
                currFlowTaskResult = flowTaskResult;
                currFlowTaskResult.setFlowId(flowId);
                break;
            }
        }

        //获取流程待审批人
        Map<String, CustomFlowVarDto> agginessMap = (Map<String, CustomFlowVarDto>) flowProcessVarMap.get(SystemConstant.AGGINESS_MAP);
        if (Objects.nonNull(tfFlowApprove.getTransferAssignee())) {
            agginessMap = JacksonUtil.readJson(tfFlowApprove.getTransferAssignee(), Map.class);
        }

        //获取流程审批历史
        List<TFFlowLogResult> tfFlowLogResultList = tfFlowLogService.findByObjectId(tfFlowJoinList.get(0).getObjectId());
        if (Objects.nonNull(tfFlowLogResultList) && tfFlowLogResultList.size() > 0) {
            List<TFFlowViewLogResult> flowViewTaskResultList = gson.fromJson(gson.toJson(tfFlowLogResultList), new TypeToken<List<TFFlowViewLogResult>>() {
            }.getType());
            if (Objects.nonNull(currFlowTaskResult)) {
                if (currFlowTaskResult.getSetup() <= 0) {
                    return new FlowViewResult(currFlowTaskResult, flowViewTaskResultList);
                } else {
                    CustomFlowVarDto customFlowVarDto = gson.fromJson(gson.toJson(agginessMap.get(currFlowTaskResult.getTaskKey())), CustomFlowVarDto.class);
                    String nextApproveUsers = flowViewTaskResultList.get(flowViewTaskResultList.size() - 1).getPendApproveUserName();
                    if (Objects.nonNull(customFlowVarDto.getMultipleUserApproveType()) && customFlowVarDto.getMultipleUserApproveType() == CustomFlowMultipleUserApproveTypeEnum.ORDER) {
                        String[] strs = nextApproveUsers.split(",");
                        nextApproveUsers = strs[0];
                        currFlowTaskResult.setApproveUserNames(nextApproveUsers);
                    } else {
                        currFlowTaskResult.setApproveUserNames(customFlowVarDto.getApproveUserNames());
                    }
                    return new FlowViewResult(customFlowVarDto.getMultipleUserApproveType(), currFlowTaskResult, flowViewTaskResultList);
                }
            }
        }
        return new FlowViewResult(currFlowTaskResult);
    }

    /**
     * 根据流程类型获取流程节点
     *
     * @param type
     * @return
     */
    @Override
    public List<FlowTaskResult> getFlowInfoByType(TFCustomTypeEnum type) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        TFCustomFlow tfCustomFlow = tfCustomFlowService.findMaxVersion(sysUser.getSchoolId(), null, type);
        Optional.ofNullable(tfCustomFlow).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到自定义流程数据"));

        List<FlowTaskResult> flowTaskResultList = this.getFlowTaskResult(tfCustomFlow);
        Map<String, Object> flowProcessVarMap = this.getFlowProcessVarMap(tfCustomFlow);
        Map<String, CustomFlowVarDto> agginessMap = (Map<String, CustomFlowVarDto>) flowProcessVarMap.get(SystemConstant.AGGINESS_MAP);
        Optional.ofNullable(tfCustomFlow).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到自定义流程数据"));

        Gson gson = new Gson();
        for (FlowTaskResult f : flowTaskResultList) {
            CustomFlowVarDto customFlowVarDto = gson.fromJson(gson.toJson(agginessMap.get(f.getTaskKey())), CustomFlowVarDto.class);
            if (Objects.nonNull(customFlowVarDto)) {
                f.setApproveUserNames(customFlowVarDto.getApproveUserNames());
            }
        }
        return flowTaskResultList;
    }

    /**
     * 获取流程变量
     *
     * @param o
     * @return
     */
    protected Map<String, Object> getFlowProcessVarMap(Object o) {
        Optional.ofNullable(o).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到自定义流程数据"));
        Map<String, Object> varMap = null;
        if (o instanceof TFCustomFlow) {
            o = (TFCustomFlow) o;
            varMap = JacksonUtil.readJson(((TFCustomFlow) o).getFlowProcessVar(), Map.class);
        } else if (o instanceof TFCustomFlowEntity) {
            o = (TFCustomFlowEntity) o;
            varMap = JacksonUtil.readJson(((TFCustomFlowEntity) o).getFlowProcessVar(), Map.class);
        }
        Optional.ofNullable(varMap).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("自定义流程变量数据为空"));
        Map<String, Object> flowProcessVarMap = (Map<String, Object>) varMap.get(SystemConstant.FLOW_PROCESS_VAR_MAP);
        Optional.ofNullable(flowProcessVarMap).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("自定义流程节点变量数据为空"));
        return flowProcessVarMap;
    }

    /**
     * 开始任务节点
     *
     * @return
     */
    protected StartEvent createStartEvent() {
        StartEvent startEvent = new StartEvent();
        startEvent.setId(CustomFlowTypeEnum.START.getId());
        startEvent.setName(CustomFlowTypeEnum.START.getTitle());
        return startEvent;
    }

    /**
     * 结束任务节点
     *
     * @return
     */
    protected EndEvent createEndEvent() {
        EndEvent endEvent = new EndEvent();
        endEvent.setId(CustomFlowTypeEnum.END.getId());
        endEvent.setName(CustomFlowTypeEnum.END.getTitle());
        return endEvent;
    }

    /**
     * @param id       对应我们画流程图中节点任务id
     * @param name     节点任务名称
     * @param assignee 任务的执行者(这一块自行决定是否添加每一环节的执行者，若是动态分配的话，可以不用传值)
     * @return
     */
    protected UserTask createUserTask(String id, String name, String assignee) {
        UserTask userTask = new UserTask();
        userTask.setName(name);
        userTask.setId(id);
        userTask.setAssignee(assignee);
        return userTask;
    }

    /**
     * @param id                                    对应我们画流程图中节点任务id
     * @param name                                  节点任务名称
     * @param assignees                             任务的执行者(这一块自行决定是否添加每一环节的执行者，若是动态分配的话，可以不用传值)
     * @param customFlowMultipleUserApproveTypeEnum
     * @return
     */
    protected UserTask createUserTask(String id, String
            name, List<String> assignees, CustomFlowMultipleUserApproveTypeEnum customFlowMultipleUserApproveTypeEnum) {
        UserTask userTask = new UserTask();
        userTask.setName(name);
        userTask.setId(id);

        boolean isSequential = false;
        if (customFlowMultipleUserApproveTypeEnum == CustomFlowMultipleUserApproveTypeEnum.ORDER) {
            userTask = multiWorkFlow.createMultiInstanceLoopCharacteristics(userTask, !isSequential);
        } else if (customFlowMultipleUserApproveTypeEnum == CustomFlowMultipleUserApproveTypeEnum.ALL) {
            userTask = multiWorkFlow.createMultiInstanceLoopCharacteristics(userTask, isSequential);
        } else {
            userTask.setCandidateUsers(assignees);
        }
        return userTask;
    }

    /**
     * 排他网关
     *
     * @param id
     * @param name
     * @return
     */
    protected ExclusiveGateway createExclusiveGateway(String id, String name) {
        ExclusiveGateway exclusiveGateway = new ExclusiveGateway();
        exclusiveGateway.setId(id);
        exclusiveGateway.setName(name);
        return exclusiveGateway;
    }

    /**
     * 并行网关
     *
     * @param id
     * @param name
     * @return
     */
    protected ParallelGateway createParallelGateway(String id, String name) {
        ParallelGateway parallelGateway = new ParallelGateway();
        parallelGateway.setId(id);
        parallelGateway.setName(name);
        return parallelGateway;
    }

    /**
     * @param from                连线来源节点
     * @param to                  连线目标节点
     * @param id
     * @param name                连线名称(可不填)
     * @param conditionExpression 网关每一种线路走向的条件表达式
     * @return
     */
    protected SequenceFlow createSequenceFlow(String from, String to, String id, String name, String
            conditionExpression) {
        SequenceFlow flow = new SequenceFlow();
        flow.setSourceRef(from);
        flow.setTargetRef(to);
        flow.setId(id);
        flow.setName(name);
        if (Objects.nonNull(conditionExpression)) {
            flow.setConditionExpression(conditionExpression);
        }
        return flow;
    }
}